unit batchmain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TFmBatchMain = class(TForm)
    LblBatchFile: TLabel;
    EdBatchFile: TEdit;
    MmDescription: TMemo;
    BtnRunBatch: TButton;
    BtnCancelBatch: TButton;
    DlgOpenBatch: TOpenDialog;
    EdNumSpecies: TEdit;
    LblNumSpecies: TLabel;
    procedure LblBatchFileClick(Sender: TObject);
    procedure ChooseBatchFile(Sender: TObject);
    procedure BtnRunBatchClick(Sender: TObject);
    procedure BtnCancelBatchClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EdNumSpeciesChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FmBatchMain: TFmBatchMain;
  BatchFilename:string;

implementation

{$R *.DFM}

uses stypes, frontend, fileio, Options;

procedure TFmBatchMain.LblBatchFileClick(Sender: TObject);
begin
 ChooseBatchFile(Sender);
 if not (batchfilename = '') then
  FmBatchMain.ActiveControl := BtnRunBatch;
end;

procedure TFmBatchMain.ChooseBatchFile(Sender: TObject);
var
 ThisEdit: TEdit;
begin
// If the user typed directly in the edit box
 if Sender is TEdit then
  begin
   ThisEdit := Sender as TEdit;
   BatchFilename:=ThisEdit.text  // Set the batch file
  end
 else  // User clicked on the label
  begin   // Show the open file dialog
   ThisEdit := ((Sender as TLabel).Focuscontrol as TEdit);
    // First set the default filename to the current file
   DlgOpenBatch.filename := BatchFilename;
    // Show the dialog and if the user clicks OK, set the new filename
   if DlgOpenBatch.execute then
    begin
      BatchFilename := DlgOpenBatch.filename;
      ThisEdit.text := Batchfilename;
    end;
  end;

 if (BatchFilename <> '') then
   if ((sender as Tcomponent).name <> 'LblBatchFilec') or
        ((sender as Tcomponent).name <> 'EdBatchFilec') then
        BtnRunBatch.Enabled := True;
end;

procedure TFmBatchMain.BtnRunBatchClick(Sender: TObject);
var
 LogFilename:string;
 temp: double;
 RunCrashed:Boolean;
begin
 BtnRunBatch.Enabled := False;
 BtnCancelBatch.Enabled := False;
 RunCrashed := False;
 LogFilename := BatchFilename;
 ChangeExtension(LogFilename,'log');
 OpenLogFile(LogFilename, BatchFilename);
 OpenBatchFile(BatchFilename);
 try
  ReadBatchFile(paramfilename,driverfilename,outfilename,time_start,time_stop,
                    stat,FmOptions.RunOptions);
  if (Application.Title = 'sensitivity') and (paramfilename <> 'senstest.par')
    then raise Exception.Create('Invalid parameter file for sensitivity test.');
  repeat
   try
    ReadParamFile(paramfilename,ModelDef.numparam,par,
                           ModelDef.numstate,stat,temp);
    MainForm.BtnRunClick(Sender);
    WriteLogFile(LogFilename,outfilename,'Run Complete - '+ DateTimeToStr(Now));
   except
    on E: Exception do
     begin
      WriteLogFile(LogFilename, outfilename, E.Message + ' - '
        + DateTimeToStr(Now));
      if not (Application.Title = 'modelbatch') then RunCrashed := True;
     end;
   end;
  until (ReadBatchFile(paramfilename,driverfilename,outfilename,time_start,
              time_stop,stat,FmOptions.RunOptions) = False) or
        (RunCrashed);
  WriteLogFile(LogFilename,batchfilename,'Batch Job Complete.');
 finally
  CloseBatchFile;
  CloseLogFile;
  BtnRunBatch.Enabled := True;
  BtnCancelBatch.Enabled := True;
  FmBatchMain.ActiveControl := BtnRunBatch;
  if RunCrashed then Abort{raise Exception.Create('Error in batch program')};
 end;
end;

procedure TFmBatchMain.BtnCancelBatchClick(Sender: TObject);
begin
 FmBatchMain.Close;
end;

procedure TFmBatchMain.FormCreate(Sender: TObject);
begin
 FmBatchMain.ActiveControl := EdBatchFile;
end;

procedure TFmBatchMain.EdNumSpeciesChange(Sender: TObject);
begin
 MainForm.SetNumSpecies(FmBatchMain.EdNumSpecies);
end;

procedure TFmBatchMain.FormShow(Sender: TObject);
begin
 if not (pos('MEL',ModelDef.modelname) > 0) then
  begin
   LblNumSpecies.Visible := False;
   LblNumSpecies.Enabled := False;
   EdNumSpecies.Visible := False;
   EdNumSpecies.Enabled := False;
  end;
end;

end.
