unit calmain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, calframe, ExtCtrls, stypes, math, Menus;

type
  TTrtArray = array[1..maxtreatments] of integer;
  StateListArray = array[1..Maxtreatments] of Tstringlist;
  TFmCalMain = class(TForm)
    PnFileInfo: TPanel;
    PnColumnHeadings: TPanel;
    LblStateVar: TLabel;
    LblParameter: TLabel;
    LblAlpha: TLabel;
    LblBeta: TLabel;
    LblBatchFilec: TLabel;
    EdBatchFilec: TEdit;
    LblMeasFile: TLabel;
    EdMeasFile: TEdit;
    LblOutputFile: TLabel;
    EdOutputFile: TEdit;
    FrCalData: TFrCalData;
    DlgOpenMeasFile: TOpenDialog;
    DlgSaveOutput: TSaveDialog;
    btnRunCal: TButton;
    btnCloseCal: TButton;
    RgOutputInfo: TRadioGroup;
    LblTreatment: TLabel;
    MMFuzzyCal: TMainMenu;
    MIFile: TMenuItem;
    MIOpenCalset: TMenuItem;
    MISaveCalSet: TMenuItem;
    MIExit: TMenuItem;
    DlgOpenCalSet: TOpenDialog;
    DlgSaveCalSet: TSaveDialog;
    MILoop: TMenuItem;
    MINoLoop: TMenuItem;
    MILoopCont: TMenuItem;
    About1: TMenuItem;
    procedure UpdateTreatments(Sender: TObject);
    procedure EdBatchFilecExit(Sender: TObject);
    procedure EdBatchFilecKeyPress(Sender: TObject; var Key: Char);
    procedure ChooseMeasFile(Sender: TObject);
    procedure ChooseCalSetFile(Sender: TObject);
    procedure SaveCalSetFile(Sender: TObject);
    procedure EdMeasFileKeyPress(Sender: TObject; var Key: Char);
    procedure ChooseOutputFile(Sender: TObject);
    procedure EdOutputFileKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure UpdateStates(Sender: TObject);
    procedure UpdateParameters(Sender: TObject);
    procedure StoreAlphaBeta(Sender: TObject);
    procedure AlphaBetaKeyPress(Sender: TObject; var Key: Char);
    procedure btnCloseCalClick(Sender: TObject);
    procedure UpdateCalForm(Sender: TObject);
    procedure CheckCalibrationSet;
    procedure CheckforRunInfo(ntreat:integer; List:TTrtArray; var TGroup: TreatSetArray);
    procedure btnRunCalClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure GetAllMeasurements(var TGroup:TreatSetArray);
    function ParseMeasData(datastring:string; var TGroup:TreatSetarray):integer;
    procedure GetSimResults(stime: double; var Trtment: TTreatSet);
    function CalculateEndofYear(sometime: double): double;
    function GetTreatment(filename:string): integer;
    function GetNextMeasurementTime(var TGroup: TreatSetArray; nTreat:integer;
                                                    lasttime: double): double;
//    function NumberofMeasurements(var TGroup: TreatSetArray; treatment:integer;
//                                           begintime, endtime:double):integer;
    procedure CalculateNewParameters(var Trtment: TTreatSet);
    procedure CheckforMeasurements(var TGroup: TreatSetArray);
    procedure RgOutputInfoClick(Sender: TObject);
    procedure ShowProgressBar(stime, ftime: double);
    procedure CancelRun;
    procedure ReadEntireBatchFile(fname:string; var ntreat:integer; var List:TTrtArray;
            var tgroup:treatsetarray; var pfilename:string; var itime, ftime:double);
    procedure CalculateCalSetTreatGroup(ntreat:integer; List:TTrtArray;
            var TGroup:treatsetarray);
    procedure GetInitialFinalTime(ntreat:integer; List:TTrtArray;
            var TGroup:treatsetarray; var itime, ftime: double);
    procedure GetNextTimeStartStop(ntreat:integer; List:TTrtArray;
            var TGroup:treatsetarray; var btime, etime: double; ctime:double);
    function MeasurementFound(tstart, tstop: double; var Trtment:TTreatSet): Boolean;
    procedure MoveCurrentTreattoGlobals(var Trtment:Ttreatset);
    procedure MoveGlobalsToCurrentTreat(var Trtment:Ttreatset);
    procedure IntializeStates(var istates: statearray; ntreat: integer;
            List:TTrtArray; var TGroup:TreatSetArray);
    procedure GetNamesUnitsSymbols (var Tgroup: treatsetarray);
    procedure PrepareTrtforNextTimeLoop(var istates: statearray; var TrtMent:TTreatSet);
    procedure MINoLoopClick(Sender: TObject);
    procedure MILoopContClick(Sender: TObject);
    procedure PreparetoRepeat(pfilename: string; istate: statearray; var sttime, intime: double;
            ntreat:integer; var List:TTrtArray; var tgroup:treatsetarray);
    procedure About1Click(Sender: TObject);
    procedure FrCalDataClick(Sender: TObject);
    procedure FrCalDataMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    calStopRun: Boolean;
    procedure Initialize2ndOutFile(bpath: string; Trtnum: integer; var Trtment: TTreatSet);
    procedure UpdateFinalOutFile(var Trtment: TTreatSet; startt, finalt: double);
    procedure Cleanup(ntreat:integer; List:TTrtArray; var Tgroup: Treatsetarray);
    function GetNextSetIndex: integer;
    function SetExists(row: integer; var index: integer): Boolean;
  public
    { Public declarations }
    MeasFileName, CalSetFilename, CalOutFilename, WorkingDir: string;
    AllStates, AllParameters, ParameterNames, AllTreatments, ValidSetIndexes: Tstringlist;
    StateNames: StateListArray;
    CurrCalSet: calsetarray;
    TotalCalSet, NumStatesMeas: integer;
    BriefOutput, RunContinuously: Boolean;
    TreatGroup: TreatSetArray;
    AllowedChange: double;
  end;

var
  FmCalMain: TFmCalMain;

implementation

uses batchmain, frontend, equations, options, display, fileio, calculate,
  ProgressBar, LoopOptions, aboutbox;

{$R *.DFM}

procedure TFmCalMain.EdBatchFilecExit(Sender: TObject);
begin
  if WorkingDir <> 'none' then FmBatchMain.DlgOpenBatch.InitialDir := WorkingDir;
  FmBatchMain.ChooseBatchFile(Sender);
  WorkingDir := ExtractFileDir(batchfilename);
  UpdateCalForm(Sender);
end;

procedure TFmCalMain.EdBatchFilecKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = Chr(13)) then FmBatchMain.ChooseBatchFile(Sender);
end;

procedure TFmCalMain.ChooseMeasFile(Sender: TObject);
var
 ThisEdit: TEdit;
begin
 if WorkingDir <> 'none' then DlgOpenMeasFile.InitialDir := WorkingDir;
// If the user typed directly in the edit box
 if Sender is TEdit then
  begin
   ThisEdit := Sender as TEdit;
   MeasFilename:=ThisEdit.text  // Set the batch file
  end
 else  // User clicked on the label
  begin   // Show the open file dialog
   ThisEdit := ((Sender as TLabel).Focuscontrol as TEdit);
    // First set the default filename to the current file
   DlgOpenMeasFile.filename := MeasFileName;
    // Show the dialog and if the user clicks OK, set the new filename
   if DlgOpenMeasFile.execute then
    begin
      MeasFileName := DlgOpenMeasFile.filename;
      ThisEdit.text := MeasFileName;
    end;
  end;

 UpdateCalForm(Sender);
end;

procedure TFmCalMain.EdMeasFileKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = Chr(13)) then ChooseMeasFile(Sender);
end;

procedure TFmCalMain.ChooseOutputFile(Sender: TObject);
begin     // If the user typed directly in the edit box
 if WorkingDir <> 'none' then DlgSaveOutput.InitialDir := WorkingDir;
 if Sender is TEdit then  CalOutFilename := EdOutputFile.text  // Set the output file
 else   // User used the menu or clicked on the Output label
  begin  // Show the Save file dialog
    // First set the default filename to the current output file.
   DlgSaveOutput.filename := CalOutFilename;
   // Show the dialog and if the user chooses OK, set the CalOutFilename to the new file.
   if DlgSaveOutput.execute then CalOutFilename := DlgSaveOutput.filename;
  end;
 EdOutputFile.text := CalOutFilename;
 UpdateCalForm(Sender);
end;

procedure TFmCalMain.EdOutputFileKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = Chr(13)) then ChooseOutputFile(Sender);
end;

procedure TFmCalMain.FormCreate(Sender: TObject);
var
 i:integer;
 CurrCombo:TComboBox;
begin
 counts;
 TotalCalSet := 0;
 BriefOutput := True;
 RunContinuously := False;
 AllowedChange := 1;
 CalStopRun := False;
 AllStates := TStringList.Create;
 AllStates.Sorted := True;
 AllStates.Duplicates := dupError;
 for i := 1 to Maxtreatments do
  begin
    TreatGroup[i].firstwrite := True;
    StateNames[i] := TStringList.Create;
    StateNames[i].Sorted := True;
    StateNames[i].Duplicates := dupError;
  end;
 AllParameters := TStringList.Create;
 AllParameters.Sorted := True;
 AllParameters.Duplicates := dupError;
 ParameterNames := TStringList.Create;
 ParameterNames.Sorted := True;
 ParameterNames.Duplicates := dupError;
 AllTreatments := TStringList.Create;
 AllTreatments.Sorted := True;
 AllTreatments.Duplicates := dupError;

 for i := 1 to ModelDef.numstate do
  begin
    AllStates.Add(stat[i].symbol);
    with TComboBox.Create(FrCalData) do
     begin
      Parent := FrCalData;
      Name := 'CbTreatment' + inttostr(i);
      Top := 13 + (i-1)*35;
      Left := 15;
      Width := 80;
      Style := csDropDownList;
      Tag := i;
      OnChange := UpdateTreatments;
     end;
    with TComboBox.Create(FrCalData) do
     begin
      Parent := FrCalData;
      Name := 'CbState' + inttostr(i);
      Top :=  13 + (i-1)*35;
      Left := 140;
      Width := 100;
      Style := csDropDownList;
      Tag := i;
      OnChange := UpdateStates;
     end;
    with TComboBox.Create(FrCalData) do
     begin
      Parent := FrCalData;
      Name := 'CbParameter' + inttostr(i);
      Top := 13 + (i-1)*35;
      Left := 292;
      Width := 100;
      Style := csDropDownList;
      Tag := i;
      OnChange := UpdateParameters;
     end;
    with TEdit.Create(FrCalData) do
     begin
      Parent := FrCalData;
      Name := 'EdAlpha' + inttostr(i);
      Top := 13 + (i-1)*35;
      Left := 432;
      Width := 100;
      Text := '';
      Tag := i;
      OnExit := StoreAlphaBeta;
      OnKeyPress := AlphaBetaKeyPress;
     end;
    with TEdit.Create(FrCalData) do
     begin
      Parent := FrCalData;
      Name := 'EdBeta' + inttostr(i);
      Top := 13 + (i-1)*35;
      Left := 568;
      Width := 100;
      Text := '';
      Tag := i;
      OnExit := StoreAlphaBeta;
      OnKeyPress := AlphaBetaKeyPress;
     end;
  end;
 AllStates.Add(' ');
 AllParameters.Add(' ');
 AllTreatments.Add(' ');
 for i := 1 to ModelDef.numparam do AllParameters.Add(par[i].symbol);
 ParameterNames.Assign(AllParameters);
 for i := 1 to MaxTreatments do
   begin
     StateNames[i].Assign(AllStates);
     AllTreatments.Add(inttostr(i));
   end;

 ValidSetIndexes := TStringList.Create;
 ValidSetIndexes.Sorted := True;
 ValidSetIndexes.Duplicates := dupError;

 for i := 0 to Frcaldata.ComponentCount - 1 do
   begin
    if FrCalData.Components[i] is TComboBox then
      begin
        CurrCombo := FrCalData.Components[i] as TComboBox;
        if CurrCombo.Name[3] = 'S' then
          CurrCombo.items.Assign(AllStates)
        else if CurrCombo.Name[3] = 'P' then
          CurrCombo.items.Assign(AllParameters)
        else if CurrCombo.Name[3] = 'T' then
          CurrCombo.items.Assign(AllTreatments)
      end;
   end;
 WorkingDir := 'none';
end;

procedure TFmCalMain.FormDestroy(Sender: TObject);
var
 i: integer;
begin
 AllStates.Free;
 for i := 1 to MaxTreatments do StateNames[i].Free;
 AllParameters.Free;
 ParameterNames.Free;
 ValidSetIndexes.Free;
end;

procedure TFmCalMain.UpdateStates(Sender: TObject);
var
 TrtCombo, CurrCombo: TComboBox;
 i, j, index, TrtNumber, rownum, setindex: integer;
 templist: Tstringlist;
 tempstring: string;
begin
templist := Tstringlist.create;
TrtCombo := nil;
try
 templist.sorted := true;
 templist.Duplicates := dupError;
 for i := 1 to MaxTreatments do StateNames[i].Assign(AllStates);
// Remove all selected variables from the Statenames list
 rownum := 1;
 repeat
 // First determine the treatment number of this calset
  for i := 0 to FrCalData.ComponentCount - 1 do
   begin
    if FrCalData.Components[i].Tag = rownum then
     if FrCalData.Components[i] is TComboBox then
       if FrCalData.Components[i].Name[3] = 'T' then
         TrtCombo := FrCalData.Components[i] as TComboBox;
   end;
 // Remove the selected state variable from the treatment name list
  if (TrtCombo <> nil) and (TrtCombo.ItemIndex <> -1) and (TrtCombo.ItemIndex <> 0) then
   begin
    TrtNumber := strtoint(TrtCombo.Items[TrtCombo.ItemIndex]);
    for i := 0 to FrCalData.ComponentCount - 1 do
     begin
      if FrCalData.Components[i].Tag = rownum then
       if FrCalData.Components[i] is TComboBox then
        if FrCalData.Components[i].Name[3] = 'S' then
         begin
           CurrCombo := FrCalData.Components[i] as TComboBox;
           if CurrCombo.itemindex <> -1 then
            begin
             if StateNames[TrtNumber].IndexOf(CurrCombo.Items[CurrCombo.ItemIndex]) = -1 then
               Exception.Create('State Variable, ' + CurrCombo.Items[CurrCombo.ItemIndex]
                 + ', already selected for this treatment.');
               StateNames[TrtNumber].Delete(StateNames[TrtNumber].
                    IndexOf(CurrCombo.Items[CurrCombo.ItemIndex]));
            end;
         end;
     end;
   end;
  rownum := rownum + 1;
  TrtCombo := nil;
 until rownum = ModelDef.numstate;

// Update each combobox with unselected states and store current selection in CurrCalSet
 rownum := 1;
 repeat
 // First determine the treatment number of this calset
  for i := 0 to FrCalData.ComponentCount - 1 do
   begin
    if FrCalData.Components[i].Tag = rownum then
     if FrCalData.Components[i] is TComboBox then
       if FrCalData.Components[i].Name[3] = 'T' then
         TrtCombo := FrCalData.Components[i] as TComboBox;
   end;
 // Then update the state combobox if a treatment has been selected
  if (TrtCombo.ItemIndex <> -1) and (TrtCombo.ItemIndex <> 0) then
   begin
    TrtNumber := strtoint(TrtCombo.Items[TrtCombo.ItemIndex]);
    for i := 0 to Frcaldata.ComponentCount - 1 do
    begin
     if FrCalData.Components[i].Tag = rownum then
      if FrCalData.Components[i] is TComboBox then
       if FrCalData.Components[i].Name[3] = 'S' then
        begin
         CurrCombo := FrCalData.Components[i] as TComboBox;
          if (CurrCombo.itemindex <> -1) then
           begin
            if not SetExists(rownum, setindex) then
             begin
              setindex := GetNextSetIndex;
              CurrCalSet[setindex].rownum := CurrCombo.tag;
             end;
            if CurrCombo.ItemIndex = 0 then
             begin
               CurrCalSet[setindex].State := ' ';
             end
            else
             begin
              tempstring := CurrCombo.Items[CurrCombo.ItemIndex];
              CurrCalSet[setindex].State := tempstring;
              index := FmCalculate.GetArrayIndex(vtstate,tempstring);
              CurrCalSet[setindex].StateIndex := index;
              templist.assign(StateNames[TrtNumber]);
              templist.add(tempstring);
              CurrCombo.items.Assign(templist);
              j := CurrCombo.Items.IndexOf(tempstring);
              CurrCombo.ItemIndex := j;
              CurrCalSet[setindex].rownum := rownum;
             end;
           end
          else           // Nothing selected in this combobox
           begin
            CurrCombo.items.Assign(StateNames[TrtNumber]);
           end;
        end;
    end;
   end;
  rownum := rownum + 1;
 until rownum = ModelDef.numstate;
finally
// CalStopRun := False;         necessary? fix
 templist.Free;
 UpdateCalForm(Sender);
end;
end;

procedure TFmCalMain.UpdateParameters(Sender: TObject);
var
 CurrCombo: TComboBox;
 i, j, index, rownum, setindex: integer;
 templist: Tstringlist;
 tempstring: string;
begin
templist := Tstringlist.create;
try
 templist.sorted := true;
 templist.Duplicates := dupError;
 ParameterNames.Assign(AllParameters);
// Remove all selected variables from the ParameterNames list
 rownum := 1;
 repeat
  for i := 0 to FrCalData.ComponentCount - 1 do
   begin
    if FrCalData.Components[i].tag = rownum then
     if FrCalData.Components[i] is TComboBox then
      begin
       if FrCalData.Components[i].Name[3] = 'P' then
        begin
         CurrCombo := FrCalData.Components[i] as TComboBox;
         if CurrCombo.itemindex <> -1 then
          ParameterNames.Delete(ParameterNames.IndexOf(CurrCombo.Items[CurrCombo.ItemIndex]));
        end;
      end;
   end;
  rownum := rownum + 1;
 until rownum = ModelDef.numstate;

// Update each combobox with unselected parameters
 rownum := 1;
 repeat
  for i := 0 to Frcaldata.ComponentCount - 1 do
   begin
    if FrCalData.Components[i].tag = rownum then
     if FrCalData.Components[i] is TComboBox then
      begin
       if FrCalData.Components[i].Name[3] = 'P' then
        begin
         CurrCombo := FrCalData.Components[i] as TComboBox;
         if (CurrCombo.ItemIndex <> -1) and (CurrCombo.ItemIndex <> 0) then   // Include currently selected parameter
           begin
             if not SetExists(rownum, setindex) then
             begin
              setindex := GetNextSetIndex;
              CurrCalSet[setindex].rownum := CurrCombo.tag;
             end;
             if CurrCombo.ItemIndex = 0 then
              begin
                CurrCalSet[setindex].Parameter := ' ';
              end
             else
              begin
               tempstring := CurrCombo.Items[CurrCombo.ItemIndex];
               CurrCalSet[setindex].Parameter := tempstring;
               index := FmCalculate.GetArrayIndex(vtparameter,tempstring);
               CurrCalSet[setindex].ParamIndex := index;
               templist.assign(ParameterNames);
               templist.add(tempstring);
               CurrCombo.items.Assign(templist);
               j := CurrCombo.Items.IndexOf(tempstring);
               CurrCombo.ItemIndex := j;
               CurrCalSet[setindex].rownum := rownum;
              end;
           end
         else           // Nothing selected in this combobox yet
           begin
             CurrCombo.items.Assign(ParameterNames);
           end;
        end;
      end;
   end;
  rownum := rownum + 1;
 until rownum = ModelDef.numstate;
finally
 templist.Free;
 UpdateCalForm(Sender);
end;
end;

procedure TFmCalMain.StoreAlphaBeta(Sender: TObject);
var
 CurrEdit: TEdit;
 setindex: integer;
begin
 if Sender is TEdit then
  begin
   CurrEdit := Sender as TEdit;
   try
    if CurrEdit.Text <> '' then
     if CurrEdit.Name[3] = 'A' then
      begin
       if (not SetExists(CurrEdit.tag, setindex)) then
        begin
         setindex := GetNextSetIndex;
         CurrCalSet[setindex].rownum := CurrEdit.Tag;
        end;
       if strtofloat(CurrEdit.Text) <> 0 then
         CurrCalSet[setindex].Alpha := strtofloat(CurrEdit.text)
       else
         CurrCalSet[setindex].Alpha := 0;
      end
     else
      begin
       if (not SetExists(CurrEdit.tag, setindex)) then
        begin
         setindex := GetNextSetIndex;
         CurrCalSet[setindex].rownum := CurrEdit.Tag;
        end;
       if strtofloat(CurrEdit.Text) <> 0 then
         CurrCalSet[setindex].Beta := strtofloat(CurrEdit.text)
       else
         CurrCalSet[setindex].Beta := 0;
      end;
   except
    on EConvertError do
     begin
      MessageDlg('Invalid entry, please reenter.', mtError, [mbOK], 0);
      CurrEdit.SelectAll;
      FmCalMain.ActiveControl := CurrEdit;
     end;
   end;
  end;
 UpdateCalForm(Sender);
end;

procedure TFmCalMain.AlphaBetaKeyPress(Sender: TObject; var Key: Char);
begin
 if (Key = Chr(13)) then StoreAlphaBeta(Sender);
end;

procedure TFmCalMain.btnCloseCalClick(Sender: TObject);
begin
 FmCalMain.Close;
end;

procedure TFmCalMain.UpdateCalForm(Sender: TObject);
var
 i: integer;
begin
 TotalCalSet := 0;
 CheckCalibrationSet;
 for i := 1 to ModelDef.numstate do
   if CurrCalSet[i].ValidSet then TotalCalSet := TotalCalSet + 1;
 if (EdBatchFilec.Text <> '') and (EdMeasFile.Text <> '') and
  (EdOutputFile.Text <> '') and (TotalCalSet <> 0) then BtnRunCal.Enabled := True;
end;

procedure TFmCalMain.CheckCalibrationSet;
var
 i, setindex: integer;
begin
 for i := 1 to ModelDef.numstate do
  begin
    if (CurrCalSet[i].Treatment <> 0) and (CurrCalSet[i].State <> ' ') and
       (CurrCalSet[i].Parameter <> ' ') and
       (CurrCalSet[i].Alpha <> 0) and (CurrCalSet[i].Beta <> 0) then
     CurrCalSet[i].ValidSet := True
    else
     CurrCalSet[i].ValidSet := False;
    if (CurrCalSet[i].Treatment = 0) and (CurrCalSet[i].State = ' ') and
       (CurrCalSet[i].Parameter = ' ') and
       (CurrCalSet[i].Alpha = 0) and (CurrCalSet[i].Beta = 0) then
     begin
       if SetExists(CurrCalSet[i].rownum, setindex) then
        begin
          CurrCalSet[i].rownum := 0;
          ValidSetIndexes.Delete(ValidSetIndexes.Indexof(inttostr(i)));
        end;
     end;
  end;
end;

procedure TFmCalMain.btnRunCalClick(Sender: TObject);
var
 initialtime, finaltime, MaxChange: double;
 i, NumTreatments: integer;
 trtlist: TTrtArray;
 batchpath, origparamfile: string;
 initialstates: statearray;
begin
 FmCalMain.Enabled := False;
 btnRunCal.Enabled := False;
 btnCloseCal.Enabled := False;
 GetNamesUnitsSymbols(TreatGroup);

 batchpath := ExtractFileDir(batchfilename);
 ReadEntireBatchFile(batchfilename, NumTreatments, Trtlist, TreatGroup, paramfilename,
        initialtime, finaltime);
 GetAllMeasurements(TreatGroup);

// Check if have everything
 CheckforMeasurements(TreatGroup);   // Makes sure have a measurement for each calset.
 CheckforRunInfo(NumTreatments, TrtList, TreatGroup);  // Makes sure have runinfo for each calset.
 CalculateCalSetTreatGroup(NumTreatments, TrtList, TreatGroup); // Matches each calset with the appropriate treatment

// Initialize
 GetInitialFinalTime(NumTreatments, TrtList, TreatGroup, initialtime, finaltime);
 GetNextTimeStartStop(NumTreatments, TrtList, TreatGroup, time_start, time_stop, initialtime);
 Showprogressbar(initialtime, finaltime);
 origparamfile := paramfilename;
 paramfilename := 'tempparfile.par';
 ReadParamFile(origparamfile,ModelDef.numparam,par,ModelDef.numstate,stat,
    Mainform.currentresid);
 WriteParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,stat,
    Mainform.currentresid);
 ReadParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,stat,
    Mainform.currentresid);
 IntializeStates(initialstates, NumTreatments, TrtList, TreatGroup);

 OpenCalOutFile(CalOutFilename);
 try try
 repeat
 MaxChange := 0;
 while time_stop <= finaltime do
  begin
   for i := 1 to NumTreatments do
    begin
     if (MeasurementFound(time_start, time_stop, TreatGroup[TrtList[i]])) and
        (TreatGroup[TrtList[i]].timestop >= time_stop) then
      begin
       Initialize2ndOutFile(batchpath, TrtList[i], TreatGroup[TrtList[i]]);
       MoveCurrentTreattoGlobals(TreatGroup[TrtList[i]]);
       MainForm.BtnRunClick(FmCalMain);    // Runs until end of year after measurement time
       MoveGlobalsToCurrentTreat(TreatGroup[TrtList[i]]);
       // Calculate new parameter values
       GetSimResults(time_stop, TreatGroup[TrtList[i]]);
       CalculateNewParameters(TreatGroup[TrtList[i]]);
       // Write out results
       WriteCalOutFile(time, TrtList[i], TotalCalSet, CurrCalSet);
       // Update second output file if necessary
       if TreatGroup[TrtList[i]].CalOptions.UseSecondOutFile then
             UpdateFinalOutFile(TreatGroup[TrtList[i]], time_start, finaltime);
       // Update everything for next loop
       PrepareTrtforNextTimeLoop(initialstates, TreatGroup[TrtList[i]]);
       Application.ProcessMessages;
       if calstopRun then break;
      end;
    end;
   if calstopRun then break;
   FmProgress.PbRunStatus.Position :=
       (round(time_stop - initialtime)*100) div round(finaltime - initialtime);
   GetNextTimeStartStop(NumTreatments, TrtList, TreatGroup, time_start, time_stop, time);
//   Application.ProcessMessages;
  end;
 if calstopRun then break;
 if RunContinuously then
  begin
   PreparetoRepeat(paramfilename, initialstates, time_start, initialtime, numtreatments,
     TrtList, TreatGroup);
   GetNextTimeStartStop(NumTreatments, TrtList, TreatGroup, time_start, time_stop, initialtime);
   FmProgress.PbRunStatus.Position :=
       (round(time_stop - initialtime)*100) div round(finaltime - initialtime);
  end;
 for i := 1 to TotalCalSet do
   begin
     MaxChange := max(MaxChange,
        CurrCalSet[i].DeltaPar/(Par[CurrCalSet[i].ParamIndex].value-CurrCalSet[i].DeltaPar));
   end;
 Cleanup(Numtreatments, TrtList, TreatGroup);
 until (not RunContinuously) or (MaxChange < AllowedChange);
 finally
  Cleanup(Numtreatments, TrtList, TreatGroup);    // Needs to be here in case of an exception otherwise if rerun, code is messed up.
  calstoprun := False;
  FmCalMain.Enabled := True;
  btnRunCal.Enabled := True;
  btnCloseCal.Enabled := True;
  CloseCalOutFile;
  FmProgress.Hide;
 end;
 except
   CloseOutputFile2;
 //  on E: Exception do
{   begin
   if RunningInteractive then
    begin
     FmTrouble.MmError.Lines.Clear;
     FmTrouble.MmError.Lines.Add(E.Message);
     FmTrouble.Memo1.Visible := False;
     FmTrouble.Memo1.Enabled := False;
     FmTrouble.ShowModal;
    end
    else    }
    raise;
   //end;
 end;
end;

procedure TFmCalMain.FormShow(Sender: TObject);
begin
 Mainform.RunningInteractive := False;
end;

{ Reads the measurements from the measured data file into the appropriate treatment
  in the treatment group array. }
procedure TFmCalMain.GetAllMeasurements(var TGroup:TreatSetArray);
var
 tempstring: string;
 i, j, k, tnum: integer;
begin
 for i := 1 to maxtreatments do
  begin
    TGroup[i].currmeasurement := 1;
    TGroup[i].TotalMeasurements := 0;
    for j := 1 to maxresults do
      begin
        for k := 1 to maxstate do TGroup[i].measdata[j].IndexList[k] := 0;
      end;
  end;
 // currmeasurement is set in MeasurementFound and used by getsimresults
 OpenMeasFile(measFilename);

 try
   if not ReadMeasFile(tempstring)then
     raise Exception.Create('Error reading measured data.');
   repeat
     tnum := ParseMeasData(tempstring, TGroup);
     if tnum > maxtreatments then
       raise Exception.Create('Invalid treatment number. ' +
           ' Treatment number in measurement file is greater than maximum.');
     if TGroup[tnum].TotalMeasurements + 1 > maxresults then
       raise Exception.Create('Too many measured data. ' +
           ' Increase maxresults in stypes.pas and repeat calibration run.');
   until ReadMeasFile(tempstring) = false;
 finally
   CloseMeasFile;
 end;
end;

{ Divides the string in datastring into measured data and stores it in Tgroup.
  Returns the treatment number. }
function TFmCalMain.ParseMeasData(datastring:string; var TGroup:TreatSetarray):integer;
var
 numbegin, numend, index, i, treatmentnum: integer;
 tempstring:string;
begin
 RemoveSpaces(datastring);

 numbegin := 1;
 numend := pos(',',datastring);
 tempstring := copy(datastring,numbegin,numend-numbegin+1);
 delete(datastring,numbegin,numend);
 delete(tempstring,pos(',',tempstring),1);
 treatmentnum := strtoint(tempstring);
 ParseMeasData := Treatmentnum;

 TGroup[treatmentnum].TotalMeasurements := TGroup[treatmentnum].TotalMeasurements + 1;
 numend := pos(',',datastring);
 tempstring := copy(datastring,numbegin,numend-numbegin+1);
 delete(datastring,numbegin,numend);
 delete(tempstring,pos(',',tempstring),1);
 TGroup[treatmentnum].measdata[TGroup[treatmentnum].TotalMeasurements].Time :=
     Strtoint(tempstring);

 NumStatesMeas := 0;
 i := 1;
 repeat
   numend := pos(',',datastring);
   tempstring := copy(datastring,numbegin,numend-numbegin+1);
   delete(datastring,numbegin,numend);
   delete(tempstring,pos(',',tempstring),1);
   index := FmCalculate.GetArrayIndex(vtstate,tempstring);
   numend := pos(',',datastring);
   if numend <> 0 then
    begin
     tempstring := copy(datastring,numbegin,numend-numbegin+1);
     delete(datastring,numbegin,numend);
     delete(tempstring,pos(',',tempstring),1);
    end
   else
    begin
     tempstring := datastring;
    end;
   TGroup[treatmentnum].measdata[TGroup[treatmentnum].TotalMeasurements].IndexList[i] := index;
   TGroup[treatmentnum].measdata[TGroup[treatmentnum].TotalMeasurements].States[index].value :=
     strtofloat(tempstring);
   NumStatesMeas := NumStatesMeas + 1;
   i := i + 1;
 until pos(',',datastring) = 0;
end;

{ Loads the simulation results for the current treatment into the display form
  and then copies the simulated results to derivstates so that the new parameter
  values can be calculated. }
procedure TFmCalMain.GetSimResults(stime:double; var Trtment: TTreatSet);
var
 j, colnum, rownum: integer;

function GetColumnNumber(name:string):integer;
var
 i, colnum:integer;
 tempstring:string;
begin
 i := 1;
 colnum := 0;
 repeat
   tempstring := FmDisplayData.SgModelOutput.Cells[i,1];
   tempstring := trim(tempstring);
   if tempstring = name then colnum := i;
   i := i + 1;
 until (colnum <> 0) or (i > FmDisplayData.SgModelOutput.Colcount - 1);
 result := colnum;
end;

function GetRowNumber(Time:string):integer;
var
 i, rownum:integer;
 tempstring:string;
begin
 i := 1;
 rownum := 0;
 repeat
   tempstring := FmDisplayData.SgModelOutput.Cells[0,i];
   tempstring := trim(tempstring);
   if tempstring = Time then rownum := i;
   i := i + 1;
 until (rownum <> 0) or (i > FmDisplayData.SgModelOutput.rowcount - 1);
 result := rownum;
end;

begin
 FmDisplayData.Filename := Trtment.outputfilename;
 FmDisplayData.UpdateStringGrid;
 Trtment.derivstates.Time2 := Trtment.measdata[Trtment.currmeasurement].Time;
// if BriefOutput or Trtment.CalOptions.UseSecondOutFile then
//   rownum := Trtment.measdata[Trtment.currmeasurement].Time - 1700 {- (round(stime/365) - 1)*365 }+ 3 + 1; //ses
// else
//   rownum := round(Trtment.measdata[Trtment.currmeasurement].Time + 3);
 rownum := GetRowNumber(floattostr(Trtment.measdata[Trtment.currmeasurement].Time));
 for j := 1 to NumStatesMeas do
   begin
     colnum := GetColumnNumber(
          stat[Trtment.measdata[Trtment.currmeasurement].IndexList[j]].name);
     Trtment.simdata[Trtment.currmeasurement].States
          [Trtment.measdata[Trtment.currmeasurement].IndexList[j]].value :=   // col, row
          strtofloat(FmDisplayData.SgModelOutput.Cells[colnum,rownum]);
     Trtment.derivstates.states2[Trtment.measdata[Trtment.currmeasurement].
          IndexList[j]].value :=
          strtofloat(FmDisplayData.SgModelOutput.Cells[colnum,rownum]);
   end;
end;

function TFmCalMain.GetTreatment(filename:string):integer;
var
 num: integer;
begin
 lowercase(filename);
 num := pos('_t',filename);
 num := num + 2;
 try
  GetTreatment := strtoint(filename[num]);
 except
  GetTreatment := 0;
  raise;
 end;
end;

function TFmCalMain.CalculateEndofYear(sometime: double): double;
var
 i: integer;
 EndofYear: double;
begin
 i := 1;
 repeat
  EndofYear := 365*i;
  i := i + 1;
 until EndofYear >= sometime;
 CalculateEndofYear := EndofYear;
end;

procedure TFmCalMain.CalculateNewParameters(var Trtment: TTreatSet);
var
 j: integer;
 derivterm: double;
 oldpar: paramarray;
begin
 oldpar := par;
 for j := 1 to Trtment.NumCalSet do
  begin
    if Trtment.derivstates.time1 <> 0 then
      derivterm := (Trtment.derivstates.states2[CurrCalSet[Trtment.CalSetIndexList[j]].StateIndex].value -
        Trtment.derivstates.states1[CurrCalSet[Trtment.CalSetIndexList[j]].StateIndex].value)/
        (Trtment.derivstates.time2 - Trtment.derivstates.time1)
    else
      derivterm := 0;
    if Trtment.measdata[Trtment.currmeasurement].
         States[CurrCalSet[Trtment.CalSetIndexList[j]].StateIndex].Value = 0 then
      CurrCalSet[Trtment.CalSetIndexList[j]].DeltaPar := 0
    else
      CurrCalSet[Trtment.CalSetIndexList[j]].DeltaPar :=
        CurrCalSet[Trtment.CalSetIndexList[j]].alpha*
        (Trtment.simdata[Trtment.currmeasurement].
        States[CurrCalSet[Trtment.CalSetIndexList[j]].StateIndex].Value -
        Trtment.measdata[Trtment.currmeasurement].
        States[CurrCalSet[Trtment.CalSetIndexList[j]].StateIndex].Value) +
        CurrCalSet[Trtment.CalSetIndexList[j]].beta*derivterm;
    Par[CurrCalSet[Trtment.CalSetIndexList[j]].ParamIndex].value :=
        Par[CurrCalSet[Trtment.CalSetIndexList[j]].ParamIndex].value
        + CurrCalSet[Trtment.CalSetIndexList[j]].DeltaPar;
 // If new parameter has a different sign then original value, raise exception
    if (((Par[CurrCalSet[Trtment.CalSetIndexList[j]].ParamIndex].value > 0) xor
          (oldPar[CurrCalSet[Trtment.CalSetIndexList[j]].ParamIndex].value > 0))) then
     begin
       raise Exception.Create('Parameter, ' +
           Par[CurrCalSet[Trtment.CalSetIndexList[j]].ParamIndex].symbol
           + ', changed sign. Run aborted.');
     end;
  end;
end;

{ Checks to be sure there's a measurement for each state variable chosen in the CalSet.}
procedure TFmCalMain.CheckforMeasurements(var TGroup: TreatSetArray);
var
 i, j, tnum: integer;
 MeasurementFound: Boolean;
begin
 for i := 1 to TotalCalSet do
  begin
   MeasurementFound := False;
   tnum := CurrCalSet[i].treatment;
   for j := 1 to NumStatesMeas do
    begin
      if TGroup[tnum].measdata[TGroup[tnum].CurrMeasurement].IndexList[j] =
        CurrCalSet[i].StateIndex then
        MeasurementFound := True;
    end;
   if not MeasurementFound then
     raise Exception.Create('No measurement found for state variable, ' +
      CurrCalSet[i].state + '. Run aborted.');
  end;
end;

procedure TFmCalMain.RgOutputInfoClick(Sender: TObject);
begin
 if RgOutputInfo.ItemIndex = 0 then
   BriefOutput := True
 else
   BriefOutput := False;
end;

procedure TFmCalMain.UpdateFinalOutFile(var Trtment: TTreatSet; startt, finalt: double);
var
 tempstate: statearray;
 tempprocess: processarray;
 tempdrive: drivearray;
 temptime: double;
 stepsperfullstep: integer;
begin
try
 stepsperfullstep := round(fullstep/Trtment.Options.Time_step);
 OpenOutputFile(Trtment.outputfilename,ModelDef.numstate, tempstate, ModelDef.numprocess,
        tempprocess, flRead);
 ReadOutputFile(temptime, ModelDef.numdrive, tempdrive, ModelDef.numstate,
        tempstate, ModelDef.numprocess, tempprocess);   // Reads in time=0 so counting below works correctly
 if Trtment.FirstWrite and (not Trtment.CalOptions.OutputEORonly) and
    (Trtment.CalOptions.OrigOutputOffset = FmOptions.DefaultRunOptions.Outputoffset) then
   begin
     WriteOutputFile2(temptime, ModelDef.numdrive, tempdrive, ModelDef.numstate,
        tempstate, ModelDef.numprocess,tempprocess);  // Writes time=0 to second outfile
     Trtment.FirstWrite := False;
   end;
 repeat
  repeat
    Trtment.CalOptions.caloutcounter := Trtment.CalOptions.caloutcounter + 1;
    ReadOutputFile(temptime, ModelDef.numdrive, tempdrive, ModelDef.numstate,
             tempstate, ModelDef.numprocess, tempprocess);
  until (Trtment.CalOptions.caloutcounter = Trtment.CalOptions.caloutputfreq {+ FmOptions.RunOptions.Time_step}) or eof(outfile); //(temptime >= time_stop)
  if not Trtment.CalOptions.OutputEORonly then
    begin
      if (temptime >= startt) and (temptime >= Trtment.CalOptions.origoutputoffset) then
        begin

          if stepsperfullstep <> fullstep then
             WriteOutputFile2(temptime-(stepsperfullstep-1)*Trtment.Options.time_step,
                  ModelDef.numdrive, tempdrive, ModelDef.numstate, tempstate,
                  ModelDef.numprocess,tempprocess)
          else
             WriteOutputFile2(temptime, ModelDef.numdrive, tempdrive,
                  ModelDef.numstate, tempstate, ModelDef.numprocess,tempprocess);
          Trtment.CalOptions.caloutcounter := 0;
        end
      else
        begin
          Trtment.CalOptions.caloutcounter := 0;
        end;
    end
  else    // Output EOR only
    begin
      if temptime >= finalt then
        WriteOutputFile2(temptime, ModelDef.numdrive, tempdrive,
                 ModelDef.numstate, tempstate, ModelDef.numprocess,tempprocess);
        Trtment.CalOptions.caloutcounter := 0;
    end;
 until temptime >= time_stop;
 Trtment.CalOptions.AppendFinalOutputFile := True;
finally
 CloseOutputFile;
 CloseOutputFile2;
end;
end;

procedure TFmCalMain.ShowProgressBar(stime, ftime: double);
begin
  FmProgress.Caption := 'Fuzzy calibrator run progress';
  FmProgress.LblBegin.Caption := floattostr(stime);
  FmProgress.LblEnd.Caption := floattostr(ftime);
  FmProgress.CancelProcedure := CancelRun;
  FmProgress.Show;
end;

procedure TFmCalMain.CancelRun;
begin
  calstopRun := True;
end;

procedure TFmCalMain.UpdateTreatments(Sender: TObject);
var
 CurrCombo: TComboBox;
 setindex: integer;
begin
 if Sender is TComboBox then
  begin
    CurrCombo := Sender as TComboBox;
    if not SetExists(CurrCombo.tag, setindex) then
     begin
      setindex := GetNextSetIndex;
      CurrCalSet[setindex].rownum := CurrCombo.tag;
     end;
    if CurrCombo.ItemIndex <> 0 then
      CurrCalSet[setindex].Treatment := strtoint(CurrCombo.Items[CurrCombo.ItemIndex])
    else
      CurrCalSet[setindex].Treatment := 0;
  end;
 UpdateStates(Sender);
 UpdateCalForm(Sender);
end;

{ Makes sure there is simulation information for the treatment of each valid
  calibration set. }
procedure TFmCalMain.CheckforRunInfo(ntreat: integer; List:TTrtArray; var TGroup: TreatSetArray);
var
 i, j: integer;
begin
 for i := 1 to TotalCalSet do
  begin
    for j := 1 to ntreat do
     begin
       if CurrCalSet[i].Treatment = List[j] then
         CurrCalSet[i].HaveRunInfo := True;
     end;
    if not CurrCalSet[i].HaveRunInfo then
     raise Exception.Create('No run information found for treatment ' +
          inttostr(CurrCalSet[i].Treatment) + '. Calibration run aborted.');
  end;
end;

function TFmCalMain.GetNextMeasurementTime(var TGroup: TreatSetArray; nTreat:integer;
                                                    lasttime: double): double;
var
 i, j: integer;
 nexttime: double;
 Tnexttimes: array[1..maxtreatments] of double;
begin
 nexttime := 0;
 for i := 1 to nTreat do
  begin
    j := 1;
    while (nexttime = 0) or (j <= TGroup[i].totalmeasurements) do
     begin
      if (TGroup[i].measdata[j].time > lasttime) then nexttime := TGroup[i].measdata[j].time;
      j := j + 1;
     end;
    Tnexttimes[i] := nexttime;
  end;
 for i := 1 to nTreat do
  begin
    nexttime := min(nexttime, Tnexttimes[i]);
  end;
 GetNextMeasurementTime := nexttime;
end;

{ Reads the entire batch file storing the run information in the variable passed
  as tgroup. }
procedure TFmCalMain.ReadEntireBatchFile(fname:string; var ntreat:integer; var List:TTrtArray;
        var tgroup:Treatsetarray; var pfilename:string; var itime, ftime:double);
var
 dfilename, ofilename, path: string;
 i, j, tnum: integer;
 start, stop:double;
 TreatComplete:array[1..maxtreatments] of Boolean;
 TempOptions: TRunOptions;
 tempstate: statearray;
begin
 path := ExtractFileDir(fname);
 tempstate := stat;
 for i := 1 to maxtreatments do TreatComplete[i] := false;    
 try
  for j := 1 to ModelDef.numstate do tempstate[j].Reset := false;
  OpenBatchFile(Fname);
  ReadBatchFile(pfilename, dfilename, ofilename, start, stop, tempstate, TempOptions);
  ntreat := 1;
  repeat
   pfilename := path + '\' + pfilename;
   dfilename := path + '\' + dfilename;
   ofilename := path + '\' + ofilename;
   tnum := GetTreatment(ofilename);
   List[ntreat] := tnum;
   if TreatComplete[tnum] then raise Exception.Create('Duplicate treatments in the batchfile.');
   if tnum > Maxtreatments then raise Exception.Create('Invalid treatment number in batchfile.');
   TGroup[tnum].drivefilename := dfilename;
   TGroup[tnum].outputfilename := ofilename;
   TGroup[tnum].timestart := start;
   TGroup[tnum].timestop := stop;
   TGroup[tnum].currstate := tempstate;
   TGroup[tnum].Options := TempOptions;
   TreatComplete[tnum] := True;
   NTreat := NTreat + 1;
  until ReadBatchFile(pfilename, dfilename, ofilename, start, stop, tempstate, TempOptions) = false;
  NTreat := NTreat - 1;
 finally
  CloseBatchFile;
 end;
end;

{ Calculates the number of calibration sets per treatment and stores the calset
  index for each matching treatment. }
procedure TFmCalMain.CalculateCalSetTreatGroup(ntreat:integer; List:TTrtArray; 
            var TGroup:treatsetarray);
var
 i:integer;
begin
 for i := 1 to ntreat do TGroup[List[i]].NumCalSet := 0;
 for i := 1 to totalcalset do
  begin
    TGroup[CurrCalSet[i].Treatment].NumCalSet :=
       TGroup[CurrCalSet[i].Treatment].NumCalSet + 1;
    TGroup[CurrCalSet[i].Treatment].CalSetIndexList[TGroup[CurrCalSet[i].Treatment].NumCalSet]
       := i;
  end;
end;

{ Determines earliest start time and the latest stop time listed in the batch file.
  Also initializes stepcounter. }
procedure TFmCalMain.GetInitialFinalTime( ntreat:integer; List:TTrtArray;
            var TGroup:treatsetarray; var itime, ftime: double);
var
 i:integer;
begin
 itime := 9999;
 ftime := -1;
 for i := 1 to ntreat do
  begin
   itime := min(itime,TGroup[List[i]].timestart);
   ftime := max(ftime,TGroup[List[i]].timestop);
   TGroup[List[i]].Options.stepcounter := FmOptions.DefaultRunOptions.stepcounter;
   TGroup[List[i]].Options.outcounter := FmOptions.DefaultRunOptions.outcounter;
  end;
end;

{ Finds the next measurement time and calculates the end of the year for that measurement.
  Returns the end of the measurement year as the stop time and the last stop time as
  the new start time. }
procedure TFmCalMain.GetNextTimeStartStop(ntreat:integer; List:TTrtArray; var TGroup:treatsetarray;
            var btime, etime:double;  ctime:double);
var
 i, j:integer;
 temptime:array[1..maxtreatments] of double;
 ttime:double;
begin
 btime := ctime;
 for j := 1 to ntreat do temptime[j] := 0;
 for i := 1 to ntreat do
  begin
   j := 0;
   repeat
     j := j + 1;
   until (TGroup[List[i]].measdata[j].Time > ctime) or (j > TGroup[List[i]].TotalMeasurements);
   if j > TGroup[List[i]].TotalMeasurements then
     raise Exception.Create('Current Time, ' + floattostr(ctime) + ', not found in measurements.');
   temptime[i] := TGroup[List[i]].measdata[j].Time;
  end;
 ttime := temptime[1];
 for i := 1 to ntreat do
  begin
   ttime := min(ttime, temptime[i]);
  end;
 etime := CalculateEndofYear(ttime);
end;

{ Checks to see if this treatment has a  measurement between the start and
  stop times. }
function TFmCalMain.MeasurementFound( tstart, tstop: double;
  var Trtment:TTreatSet): Boolean;
var
 j:integer;
begin
 MeasurementFound := False;
 for j := Trtment.currmeasurement to Trtment.TotalMeasurements do
  begin
    if (Trtment.measdata[j].Time > tstart) and (Trtment.measdata[j].Time <= tstop) then
     begin
      Trtment.currmeasurement := j;
      MeasurementFound := True;
     end;
  end;
end;

procedure TFmCalMain.MoveCurrentTreattoGlobals(var Trtment:Ttreatset);
begin
 stat := Trtment.currstate;
 drive := Trtment.currdrive;
 driverfilename := Trtment.drivefilename;
 outfilename := Trtment.outputfilename;
 FmOptions.RunOptions := Trtment.Options;
end;

procedure TFmCalMain.MoveGlobalsToCurrentTreat(var Trtment:Ttreatset);
begin
 Trtment.currstate := stat;
 Trtment.currdrive := drive;
 Trtment.Options := FmOptions.RunOptions;
end;

{ Sets name, units, and symbols for state variables and sets initial derivative
  time. Also copies the initial values for all the state variables that were in
  the parameter file into the state array for each treatment. }
procedure TFmCalMain.IntializeStates(var istates: statearray; ntreat: integer;
           List:TTrtArray; var TGroup:TreatSetArray);
var
 i, j: integer;
begin
 istates := stat;
 for i := 1 to ntreat do
   begin
    for j := 1 to ModelDef.numstate do     // Need to do these fields individually otherwise lose
     begin                                 // value of resetstate and holdconstant read in from
                                           // batch file.
      TGroup[List[i]].currstate[j].name := stat[j].name;
      TGroup[List[i]].currstate[j].units := stat[j].units;
      TGroup[List[i]].currstate[j].symbol := stat[j].symbol;
      TGroup[List[i]].currstate[j].value := stat[j].value;
     end;
     TGroup[List[i]].currdrive := drive;  // To get names, units and symbols
 //    TGroup[List[i]].Options.DiscreteStep := TGroup[List[i]].Options.Time_step;
     TGroup[List[i]].derivstates.States1 := stat;   // To get name, units and symbol.
     TGroup[List[i]].derivstates.States2 := stat;
     TGroup[List[i]].derivstates.Time1 := 0;  // Flag so CalculateNewParameters knows this is the first time through and will ignore the derivative.
   end;              
end;

procedure TFmCalMain.GetNamesUnitsSymbols (var Tgroup: treatsetarray);
var
 i, j: integer;
begin
 for i := 1 to MaxTreatments do
  begin
   for j := 1 to MaxResults do
     begin                   // To initialize names, symbols and units
       TGroup[i].simdata[j].States := stat;
       TGroup[i].measdata[j].States := stat;
     end;
  end;
end;

procedure TFmCalMain.PrepareTrtforNextTimeLoop(var istates: statearray;
          var TrtMent:TTreatSet);
var
 i: integer;
begin
 TrtMent.derivstates.time1 := TrtMent.derivstates.time2;
 TrtMent.derivstates.states1 := TrtMent.derivstates.states2;
 TrtMent.Options.stepcounter := FmOptions.DefaultRunOptions.stepcounter;
 if TrtMent.Options.ResetStates then
  for i := 1 to ModelDef.numstate do
   begin
    if Trtment.currstate[i].Reset then
       TrtMent.currstate[i].value := istates[i].value;
   end;
{ if (BriefOutput) {and (not Trtment.CalOptions.UseSecondOutFile) then
   Trtment.CalOptions.AppendFinalOutputFile := False
 else
   Trtment.CalOptions.AppendFinalOutputFile := True;            }
end;

procedure TFmCalMain.Initialize2ndOutFile(bpath: string; Trtnum: integer;
            var Trtment: TTreatSet);
var
 tempoutfilename: string;
begin
 tempoutfilename := bpath + '\tempoutput' + inttostr(trtnum) + '.out';
 if Trtment.CalOptions.UseSecondOutFile then
   begin
     OpenOutputfile2(Trtment.CalOptions.Finaloutfilename, ModelDef.numdrive,
        Trtment.currdrive,
        ModelDef.numstate, Trtment.currstate, ModelDef.numprocess, proc,
        flwrite, Trtment.CalOptions.AppendFinalOutputFile);
   end
 else if (not BriefOutput) or
         (Trtment.Options.Outputstep <> FmOptions.DefaultRunOptions.Outputstep)
         or (Trtment.Options.OutputEORonly) then
 {  if (Trtment.Options.Outputstep <> FmOptions.DefaultRunOptions.Outputstep)
      or (Trtment.Options.Outputoffset <> FmOptions.DefaultRunOptions.Outputoffset)
      or (Trtment.Options.OutputEORonly) then   }
   begin
     Trtment.CalOptions.UseSecondOutFile := True;
     Trtment.CalOptions.AppendFinalOutputFile := Trtment.Options.AppendOutputFile;
     Trtment.CalOptions.caloutcounter := FmOptions.DefaultRunOptions.outcounter;
     Trtment.CalOptions.caloutputfreq := round(Trtment.Options.Outputstep
                                 /Trtment.Options.Time_step);
     Trtment.CalOptions.FinalOutFilename := Trtment.outputfilename;
     Trtment.outputfilename := tempoutfilename;
     OpenOutputfile2(Trtment.CalOptions.FinalOutFilename, ModelDef.numdrive, Trtment.currdrive,
        ModelDef.numstate, Trtment.currstate, ModelDef.numprocess, proc,
        flwrite, Trtment.CalOptions.AppendFinalOutputFile);
     Trtment.CalOptions.OrigOutputstep := Trtment.Options.Outputstep;
     Trtment.CalOptions.origOutputOffset := Trtment.Options.OutputOffset;
     Trtment.CalOptions.OutputEORonly := Trtment.Options.OutputEORonly;
     Trtment.Options.Outputstep := FmOptions.DefaultRunOptions.Outputstep;
     Trtment.Options.OutputOffset := FmOptions.DefaultRunOptions.Outputoffset;
     Trtment.Options.OutputEORonly := FmOptions.DefaultRunOptions.OutputEORonly;
     Trtment.Options.AppendOutputFile := False;
   end;
end;

procedure TFmCalMain.Cleanup(ntreat:integer; List:TTrtArray; var Tgroup: Treatsetarray);
var
 i: integer;
begin
 for i := 1 to ntreat do
  begin
   TGroup[List[i]].CalOptions.UseSecondOutFile := False;
   TGroup[List[i]].FirstWrite := True;
  end;
end;

procedure TFmCalMain.ChooseCalSetFile(Sender: TObject);
var
 setnum, j, setindex: integer;
 CurrCombo: TComboBox;
 CurrEdit: TEdit;
begin
  // First set the default filename to the current file
 DlgOpenCalSet.filename := CalSetFilename;
  // Show the dialog and if the user clicks OK, save the filename and read in
  // the calibration set.
 if DlgOpenCalSet.execute then
  begin
   CalSetFilename := DlgOpenCalSet.filename;
   // Clear anything already in the window
   for setnum := 1 to TotalCalSet do
    begin
      setindex := strtoint(ValidSetIndexes[setnum - 1]);
      CurrCalSet[setindex].ValidSet := False;
      CurrCalSet[setindex].Treatment := 0;
      for j := 0 to FrCalData.ComponentCount - 1 do
        begin
//          if FrCalData.Components[j].Tag = setnum then
           begin
            if FrCalData.Components[j] is TComboBox then
             begin
              CurrCombo := FrCalData.Components[j] as TComboBox;
              CurrCombo.itemindex := -1;
              if CurrCombo.Name[3] = 'S' then
                 CurrCombo.items.Assign(AllStates)
              else if CurrCombo.Name[3] = 'P' then
                 CurrCombo.items.Assign(AllParameters)
             end
            else if FrCalData.Components[j] is TEdit then
             begin
              CurrEdit := FrCalData.Components[j] as TEdit;
              CurrEdit.Text := '';
             end;
           end;
        end;
    end;
   ValidSetIndexes.Clear;
   // Read new calsets from file
   ReadCalSetFile(CalSetFilename, TotalCalSet, CurrCalSet);
   // Update window with data read in
   for setnum := 1 to TotalCalSet do
    begin
      setindex := GetNextSetIndex;
      CurrCalSet[setindex].ValidSet := True;
      for j := 0 to FrCalData.ComponentCount - 1 do
        begin
          if FrCalData.Components[j].Tag = setnum then
           begin
            if FrCalData.Components[j].Name[3] = 'S' then   // State variable combo
             begin
               CurrCalSet[setindex].rownum := setnum;
               CurrCombo := FrCalData.Components[j] as TComboBox;
               CurrCombo.itemindex :=
                     CurrCombo.Items.IndexOf(CurrCalSet[setindex].State);
             end
            else if FrCalData.Components[j].Name[3] = 'P' then    // Parameter combo
             begin
               CurrCombo := FrCalData.Components[j] as TComboBox;
               CurrCombo.itemindex :=
                     CurrCombo.Items.IndexOf(CurrCalSet[setindex].Parameter);
             end
            else if FrCalData.Components[j].Name[3] = 'T' then    // Treatment combo
             begin
               CurrCombo := FrCalData.Components[j] as TComboBox;
               CurrCombo.itemindex :=
                     CurrCombo.Items.IndexOf(inttostr(CurrCalSet[setindex].Treatment));
             end
            else if FrCalData.Components[j].Name[3] = 'A' then  // Alpha Edit box
             begin
              CurrEdit := FrCalData.Components[j] as TEdit;
              CurrEdit.Text := floattostr(CurrCalSet[setindex].alpha);
             end
            else if FrCalData.Components[j].Name[3] = 'B' then    // Beta Edit box
             begin
              CurrEdit := FrCalData.Components[j] as TEdit;
              CurrEdit.Text := floattostr(CurrCalSet[setindex].beta);
             end;
           end;
        end;
    end;
   UpdateTreatments(DlgOpenCalSet);
//   UpdateStates(DlgOpenCalSet);     Not needed because updatetreatments calls updatestates
   UpdateParameters(DlgOpenCalSet);
  end;
end;

procedure TFmCalMain.SaveCalSetFile(Sender: TObject);
begin
  // First set the default filename to the current file
 DlgSaveCalSet.filename := CalSetFilename;
  // Show the dialog and if the user clicks OK, save the file.
 if DlgSaveCalSet.execute then
  begin
   CalSetFilename := DlgSaveCalSet.filename;
   WriteCalSetFile(CalSetFilename, TotalCalSet, ValidSetIndexes, CurrCalSet);
  end;
end;

procedure TFmCalMain.MINoLoopClick(Sender: TObject);
begin
  RunContinuously := False;
  MINoLoop.Checked := not RunContinuously;
end;

procedure TFmCalMain.MILoopContClick(Sender: TObject);
begin
  FmLoopOptions.Show;
end;

procedure TFmCalMain.PreparetoRepeat(pfilename: string; istate: statearray;
            var sttime, intime: double; ntreat:integer; var List:TTrtArray;
            var tgroup:treatsetarray);
var
 i, j: integer;
begin
 for i := 1 to ModelDef.numstate do stat[i].value := istate[i].value;
 WriteParamfile(pfilename, ModelDef.numparam, par, ModelDef.numstate, stat,
    Mainform.currentresid);
 sttime := intime;
 time := intime;
 for i := 1 to ntreat do
  begin
   tgroup[List[i]].currmeasurement := 1;
   tgroup[List[i]].derivstates.Time1 := 0;
   for j := 1 to ModelDef.numstate do
        tgroup[List[i]].currstate[j].value := istate[j].value;
   if tgroup[List[i]].CalOptions.UseSecondOutFile then
     tgroup[List[i]].outputfilename := tgroup[List[i]].CalOptions.FinalOutFilename;
  end;
end;

procedure TFmCalMain.About1Click(Sender: TObject);
begin
 FmAbout.LblModel.Caption := ModelDef.modelname;
 FmAbout.LblVersion.Caption := ModelDef.versionnumber;
 FmAbout.MoContact.Lines.Add(ModelDef.contactperson);
 FmAbout.MoContact.Lines.Add(ModelDef.contactaddress1);
 FmAbout.MoContact.Lines.Add(ModelDef.contactaddress2);
 FmAbout.MoContact.Lines.Add(ModelDef.contactaddress3);
 FmAbout.Label3.Caption := 'Vegomatic';
 FmAbout.Label1.Caption := '1.0.0';
 FmAbout.ShowModal;
end;

procedure TFmCalMain.FrCalDataClick(Sender: TObject);
begin
 FmCalMain.ActiveControl := FrCalData;
end;

procedure TFmCalMain.FrCalDataMouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
begin
 if WheelDelta > 0 then
  begin
   FrCalData.VertScrollBar.Position := FrCalData.VertScrollBar.Position + WheelDelta;
  end;
end;

function TFmCalMain.GetNextSetIndex: integer;
var
 i, newindex: integer;
begin
 i := 1;
 newindex := -1;
 repeat
  if (CurrCalSet[i].rownum = 0) then newindex := i;
  i := i + 1;
 until (newindex <> -1) or (i = ModelDef.numstate);
 if newindex <> -1 then ValidSetIndexes.Add(inttostr(newindex));
 GetNextSetIndex := newindex;
end;

function TFmCalMain.SetExists(row: integer; var index: integer): Boolean;
var
 i: integer;
 setfound: boolean;
begin
 setfound := false;
 i := 1;
 index := -1;
 repeat
   if CurrCalSet[i].rownum = row then
    begin
     index := i;
     setfound := true;
    end;
   i := i + 1;
 until setfound or (i = ModelDef.numstate);
 SetExists := setfound;
end;

end.

