{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;
  newyear:Boolean = false;

implementation

uses frontend, calculate, options;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'MBL GEM';
ModelDef.versionnumber := '6.4.4.3';
ModelDef.timeunit := 'days';
ModelDef.contactperson := 'Ed Rastetter or Bonnie Kwiatkowski';
ModelDef.contactaddress1 := 'Ecosystems Center - MBL';
ModelDef.contactaddress2 := '7 MBL St.';
ModelDef.contactaddress3 := 'Woods Hole, MA 02536';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 61;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is stringlength (specified in unit stypes) characters. }
 
 
with stat[1] do
 begin
    name:='C: foliar';  units:='g C m-2 ground';  symbol:='Cf';
 end;
 
with stat[2] do
 begin
    name:='C: sapwood';  units:='g C m-2 ground';  symbol:='Csw';
 end;
 
with stat[3] do
 begin
    name:='C: heartwood';  units:='g C m-2 ground';  symbol:='Chw';
 end;
 
with stat[4] do
 begin
    name:='C: root';  units:='g C m-2 ground';  symbol:='Cr';
 end;
 
with stat[5] do
 begin
    name:='N: foliar';  units:='g N m-2 ground';  symbol:='Nf';
 end;
 
with stat[6] do
 begin
    name:='N: sapwood';  units:='g N m-2 ground';  symbol:='Nsw';
 end;
 
with stat[7] do
 begin
    name:='N: heartwood';  units:='g N m-2 ground';  symbol:='Nhw';
 end;
 
with stat[8] do
 begin
    name:='N: root';  units:='g N m-2 ground';  symbol:='Nr';
 end;
 
with stat[9] do
 begin
    name:='C: labile';  units:='g C m-2 ground';  symbol:='Cl';
 end;
 
with stat[10] do
 begin
    name:='N: labile';  units:='g N m-2 ground';  symbol:='Nl';
 end;
 
with stat[11] do
 begin
    name:='C: acid soluble soil';  units:='g C m-2 ground';  symbol:='Cs';
 end;
 
with stat[12] do
 begin
    name:='C: extractive soil';  units:='g C m-2 ground';  symbol:='Ce';
 end;
 
with stat[13] do
 begin
    name:='C: acid insoluble soil';  units:='g C m-2 ground';  symbol:='Ci';
 end;
 
with stat[14] do
 begin
    name:='C: humus soil';  units:='g C m-2 ground';  symbol:='Ch';
 end;
 
with stat[15] do
 begin
    name:='N: active soil';  units:='g N m-2 ground';  symbol:='Na';
 end;
 
with stat[16] do
 begin
    name:='N: humus soil';  units:='g N m-2 ground';  symbol:='Nh';
 end;
 
with stat[17] do
 begin
    name:='NH4: total soil';  units:='g N m-2 ground';  symbol:='Nnh4';
 end;
 
with stat[18] do
 begin
    name:='NO3: total soil';  units:='g N m-2 ground';  symbol:='Nno3';
 end;
 
with stat[19] do
 begin
    name:='H2O: Root-zone';  units:='mm H2O';  symbol:='Wr';
 end;
 
with stat[20] do
 begin
    name:='H2O: Groundwater';  units:='mm H2O';  symbol:='Wg';
 end;
 
with stat[21] do
 begin
    name:='H2O: Snowpack';  units:='mm H2O';  symbol:='Ws';
 end;
 
with stat[22] do
 begin
    name:='LAI: End of Grow season';  units:='m2 leaf m-2 grd';  symbol:='Laieos';
 end;
 
with stat[23] do
 begin
    name:='Julian day';  units:='day number';  symbol:='Jday';
 end;
 
with stat[24] do
 begin
    name:='Cum. degree days; air';  units:='degree C day';  symbol:='DDa';
 end;
 
with stat[25] do
 begin
    name:='Cum. degree days; soil';  units:='degree C day';  symbol:='DDs';
 end;
 
with stat[26] do
 begin
    name:='N Uptake mean cost';  units:='g C g-1 N';  symbol:='qUnT';
 end;
 
with stat[27] do
 begin
    name:='Cum. Precipitation';  units:='mm H2O yr-1';  symbol:='PptT';
 end;
 
with stat[28] do
 begin
    name:='Cum. Interception';  units:='mm H2O yr-1';  symbol:='PptIT';
 end;
 
with stat[29] do
 begin
    name:='Cum. Evapotranspiration';  units:='mm H2O yr-1';  symbol:='EtT';
 end;
 
with stat[30] do
 begin
    name:='Cum. Discharge';  units:='mm H2O yr-1';  symbol:='QgT';
 end;
 
with stat[31] do
 begin
    name:='Cum. Water Losses';  units:='mm H2O yr-1';  symbol:='QwT';
 end;
 
with stat[32] do
 begin
    name:='Cum. N Deposition';  units:='g N m-2 yr-1';  symbol:='DnT';
 end;
 
with stat[33] do
 begin
    name:='Cum. Photosynthesis';  units:='g C m-2 yr-1';  symbol:='PsT';
 end;
 
with stat[34] do
 begin
    name:='Cum. Net Primary Prod.';  units:='g C m-2 yr-1';  symbol:='NPPT';
 end;
 
with stat[35] do
 begin
    name:='Cum. NPP Foliage';  units:='g C m-2 yr-1';  symbol:='NPPFT';
 end;
 
with stat[36] do
 begin
    name:='Cum. NPP Wood';  units:='g C m-2 yr-1';  symbol:='NPPWT';
 end;
 
with stat[37] do
 begin
    name:='Cum. NPP Root';  units:='g C m-2 yr-1';  symbol:='NPPRT';
 end;
 
with stat[38] do
 begin
    name:='Cum. translocation';  units:='g N m-2 yr-1';  symbol:='tauNT';
 end;
 
with stat[39] do
 begin
    name:='Cum. resp. plant';  units:='g C m-2 yr-1';  symbol:='RPT';
 end;
 
with stat[40] do
 begin
    name:='Cum. resp. Foliage';  units:='g C m-2 yr-1';  symbol:='RfT';
 end;
 
with stat[41] do
 begin
    name:='Cum. resp. Wood';  units:='g C m-2 yr-1';  symbol:='RwT';
 end;
 
with stat[42] do
 begin
    name:='Cum. resp. Root';  units:='g C m-2 yr-1';  symbol:='RrT';
 end;
 
with stat[43] do
 begin
    name:='Cum. litter foliar C';  units:='g C m-2 yr-1';  symbol:='lambdaCfT';
 end;
 
with stat[44] do
 begin
    name:='Cum. litter wood C';  units:='g C m-2 yr-1';  symbol:='lambdaCwT';
 end;
 
with stat[45] do
 begin
    name:='Cum. litter root C';  units:='g C m-2 yr-1';  symbol:='lambdaCrT';
 end;
 
with stat[46] do
 begin
    name:='Cum. litter foliar N';  units:='g N m-2 yr-1';  symbol:='lambdaNfT';
 end;
 
with stat[47] do
 begin
    name:='Cum. litter wood N';  units:='g N m-2 yr-1';  symbol:='lambdaNwT';
 end;
 
with stat[48] do
 begin
    name:='Cum. litter root N';  units:='g N m-2 yr-1';  symbol:='lambdaNrT';
 end;
 
with stat[49] do
 begin
    name:='Cum. litter C';  units:='g C m-2 yr-1';  symbol:='lambdaCT';
 end;
 
with stat[50] do
 begin
    name:='Cum. litter N';  units:='g N m-2 yr-1';  symbol:='lambdaNT';
 end;
 
with stat[51] do
 begin
    name:='Cum. resp. soil';  units:='g C m-2 yr-1';  symbol:='RST';
 end;
 
with stat[52] do
 begin
    name:='Cum. NEP';  units:='g C m-2 yr-1';  symbol:='NepT';
 end;
 
with stat[53] do
 begin
    name:='Cum. N uptake';  units:='g N m-2 yr-1';  symbol:='UNT';
 end;
 
with stat[54] do
 begin
    name:='Cum. N fixation';  units:='g N m-2 yr-1';  symbol:='UnfixT';
 end;
 
with stat[55] do
 begin
    name:='Cum. gross mineralizati';  units:='g N m-2 yr-1';  symbol:='MnT';
 end;
 
with stat[56] do
 begin
    name:='Cum. Immobilization';  units:='g N m-2 yr-1';  symbol:='InnT';
 end;
 
with stat[57] do
 begin
    name:='Cum. Net N min';  units:='g N m-2 yr-1';  symbol:='NminT';
 end;
 
with stat[58] do
 begin
    name:='Cum. N leaching';  units:='g N m-2 yr-1';  symbol:='QnT';
 end;
 
with stat[59] do
 begin
    name:='Jday Fol Sens = 999';  units:='Julian Day';  symbol:='Jdfs';
 end;
 
with stat[60] do
 begin
    name:='Jday Wood Sens = 999';  units:='Julian Day';  symbol:='Jdws';
 end;
 
with stat[61] do
 begin
    name:='Jday Root Sens = 999';  units:='Julian Day';  symbol:='Jdrs';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 147;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'Temp: Air mean daily';
      units       := 'oC';
      symbol       := 'Ta';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'Day length';
      units       := 'hours';
      symbol       := 'Hd';
      parameters       := 1;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='latitude';  units:='degrees';  symbol:='Deltad';
 end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Rainfall';
      units       := 'mm H2O day-1';
      symbol       := 'Pptr';
      parameters       := 1;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='critical Ta for snowfall';  units:='oC';  symbol:='Tas';
 end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Snowfall';
      units       := 'mm H2O day-1';
      symbol       := 'Ppts';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Interception';
      units       := 'mm H2O day-1';
      symbol       := 'PptI';
      parameters       := 2;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='intercept/LAI';  units:='mmH2O m2 gnd m-2 leaf day-1';  symbol:='bIf';
 end;
with par[npar + 2] do
 begin
    name:='intercept/sapwood';  units:='mm H2O m2 g-1 C day-1';  symbol:='bIsw';
 end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Available';
      units       := 'fraction';
      symbol       := 'Aw';
      parameters       := 2;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='soil H2O @ wilting';  units:='mm H2O';  symbol:='Womega';
 end;
with par[npar + 2] do
 begin
    name:='soil H2O @ field cap';  units:='mm H2O';  symbol:='Wf';
 end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'H2O: tension deficit';
      units       := 'MPa';
      symbol       := 'PsiD';
      parameters       := 2;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='tension @ wilting';  units:='MPa';  symbol:='PsiOmega';
 end;
with par[npar + 2] do
 begin
    name:='tension @ field cap';  units:='MPa';  symbol:='Psif';
 end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Canopy conductance';
      units       := 'm H2O sec-1';
      symbol       := 'gcW';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Evapotranspiration';
      units       := 'mm H2O day-1';
      symbol       := 'ET';
      parameters       := 2;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='canopy albedo';  units:='unitless';  symbol:='alphaT';
 end;
with par[npar + 2] do
 begin
    name:='boundary H2O cond';  units:='m H2O sec-1';  symbol:='gAW';
 end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Snowmelt';
      units       := 'mm H2O day-1';
      symbol       := 'Ms';
      parameters       := 3;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='forest canopy denst';  units:='fraction';  symbol:='rhot';
 end;
with par[npar + 2] do
 begin
    name:='snow albedo';  units:='fraction';  symbol:='alphas';
 end;
with par[npar + 3] do
 begin
    name:='cover corr. factor';  units:='unitless';  symbol:='gammat';
 end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'H2O: soil drainage';
      units       := 'mm H2O day-1';
      symbol       := 'DR';
      parameters       := 1;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='drainage rate';  units:='day-1';  symbol:='alphaD';
 end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'H2O: grnd discharge';
      units       := 'mm H2O day-1';
      symbol       := 'Qg';
      parameters       := 1;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='discharge rate';  units:='day-1';  symbol:='alphaQ';
 end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'NH4: Aqueous';
      units       := 'umol N L-1 H2O';
      symbol       := 'Nnh4aq';
      parameters       := 4;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='soil bulk density';  units:='g m-3';  symbol:='rhoS';
 end;
with par[npar + 2] do
 begin
    name:='soil depth';  units:='m';  symbol:='z';
 end;
with par[npar + 3] do
 begin
    name:='max NH4 sorbed';  units:='g N g-1 dry soil';  symbol:='Vsnh4max';
 end;
with par[npar + 4] do
 begin
    name:='sorbed NH4 half sat.';  units:='umol N L-1 H2O';  symbol:='etaNH4';
 end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'NO3: Aqueous';
      units       := 'umol N L-1 H2O';
      symbol       := 'Nno3aq';
      parameters       := 2;
      ptype       := ptGroup4;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max NO3 sorbed';  units:='g N g-1 dry soil';  symbol:='Vsno3max';
 end;
with par[npar + 2] do
 begin
    name:='sorbed NO3 half sat.';  units:='umol N L-1 H2O';  symbol:='etaNO3';
 end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'NH4: sorbed';
      units       := 'g N m-2 ground';
      symbol       := 'Nnh4s';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := 'NO3: sorbed';
      units       := 'g N m-2 ground';
      symbol       := 'Nno3s';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'NH4: leaching';
      units       := 'g N m-2 day-1';
      symbol       := 'QNH4';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'NO3: leaching';
      units       := 'g N m-2 day-1';
      symbol       := 'QNO3';
      parameters       := 0;
      ptype       := ptGroup4;
   end;
 
CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'C: SW Foliage Req.';
      units       := 'g C m-2';
      symbol       := 'Cswreq';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='sap wood coefficient';  units:='empirical relationship';  symbol:='bcsw1';
 end;
with par[npar + 2] do
 begin
    name:='sap wood exponent';  units:='empirical relationship';  symbol:='bcsw2';
 end;
 
CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'Canopy C conductance';
      units       := 'gC m3air ml-1 m-2 day-1';
      symbol       := 'gcC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='soil hydraulic res';  units:='sec m2 MPa mmol-1';  symbol:='omegaS';
 end;
 
CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'Leaf Area Index';
      units       := 'm2 leaf m-2 ground';
      symbol       := 'LAI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Foliage C Density';  units:='g C m-2 leaf';  symbol:='mF';
 end;
 
CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'Photosynthesis: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'Ps';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Ps leaf N conversion';  units:='unitless';  symbol:='aNf';
 end;
with par[npar + 2] do
 begin
    name:='Cl at half Ps';  units:='g C m-2';  symbol:='Clmid';
 end;
with par[npar + 3] do
 begin
    name:='Ps rate of carbo inhibit';  units:='unitless';  symbol:='epsilonPs';
 end;
 
CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'Photosyn: isotopic sig';
      units       := 'o/oo';
      symbol       := 'deltaPs';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='isotopic signature of atm';  units:='o/oo';  symbol:='deltaCa';
 end;
with par[npar + 2] do
 begin
    name:='diffusion fractionation';  units:='o/oo';  symbol:='aDelta';
 end;
with par[npar + 3] do
 begin
    name:='net carboxylation fract';  units:='o/oo';  symbol:='bDelta';
 end;
 
CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'N: Transloc. Drought';
      units       := 'g N m-2 day-1';
      symbol       := 'tauND';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Drought N trans rate';  units:='unitless';  symbol:='atauD';
 end;
 
CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'N: Translocation';
      units       := 'g N m-2 day-1';
      symbol       := 'tauNlambda';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='translocation fraction';  units:='fraction';  symbol:='atau';
 end;
 
CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'Uptake: temp response';
      units       := 'unitless';
      symbol       := 'HU';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Uptake max temp';  units:='oC';  symbol:='TUmax';
 end;
with par[npar + 2] do
 begin
    name:='Uptake opt temp';  units:='oC';  symbol:='TUopt';
 end;
with par[npar + 3] do
 begin
    name:='Uptake T curvature';  units:='oC-1';  symbol:='gammaU';
 end;
 
CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'NH4: uptake';
      units       := 'g N m-2 day-1';
      symbol       := 'UNH4';
      parameters       := 5;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max NH4 uptake';  units:='g N m-2 day-1';  symbol:='UNH4max';
 end;
with par[npar + 2] do
 begin
    name:='NH4 1/2 saturat';  units:='umol N L-1 H2O';  symbol:='kNH4';
 end;
with par[npar + 3] do
 begin
    name:='Un Sens. to Nr';  units:='m2 gnd g-1 C';  symbol:='thetaU';
 end;
with par[npar + 4] do
 begin
    name:='Nl at half Un';  units:='g N m-2';  symbol:='Nlmid';
 end;
with par[npar + 5] do
 begin
    name:='Un rate of N inhibit';  units:='unitless';  symbol:='epsilonUn';
 end;
 
CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'NO3: uptake';
      units       := 'g N m-2 day-1';
      symbol       := 'UNO3';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max NO3 uptake';  units:='g N m-2 day-1';  symbol:='UNO3max';
 end;
with par[npar + 2] do
 begin
    name:='NO3 1/2 saturat';  units:='umol N L-1 H2O';  symbol:='kNO3';
 end;
 
CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'N Uptake cost - daily';
      units       := 'g C g-1 N';
      symbol       := 'qUn';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N uptake cost coeff';  units:='gC g-1 N MJ-1';  symbol:='gUn';
 end;
 
CurrentProc := ModelDef.numstate + 30;
With proc[CurrentProc] do
   begin
      name       := 'N: Fixation daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Unfix';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Cost of N Fixation';  units:='g C g-1 N';  symbol:='qUnfix';
 end;
with par[npar + 2] do
 begin
    name:='N fixation rate';  units:='g N g-1 Cr day-1';  symbol:='gNfix';
 end;
with par[npar + 3] do
 begin
    name:='Nfix cost half sat.';  units:='g C g-1 N';  symbol:='kNfix';
 end;
 
CurrentProc := ModelDef.numstate + 31;
With proc[CurrentProc] do
   begin
      name       := 'Resp: N Fixation';
      units       := 'g C m-2 day-1';
      symbol       := 'RUnfix';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 32;
With proc[CurrentProc] do
   begin
      name       := 'Resp: temp response';
      units       := 'unitless';
      symbol       := 'HR';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Resp max temp';  units:='oC';  symbol:='TRmax';
 end;
with par[npar + 2] do
 begin
    name:='Resp opt temp';  units:='oC';  symbol:='TRopt';
 end;
with par[npar + 3] do
 begin
    name:='Resp T curvature';  units:='oC-1';  symbol:='gammaR';
 end;
 
CurrentProc := ModelDef.numstate + 33;
With proc[CurrentProc] do
   begin
      name       := 'Resp: tot metabolic fol';
      units       := 'g C m-2 day-1';
      symbol       := 'RFmet';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Met resp coefficient';  units:='g C g-1 N day-1';  symbol:='rm';
 end;
with par[npar + 2] do
 begin
    name:='Min. active Fol. C/N';  units:='g C g-1 N';  symbol:='qFmin';
 end;
 
CurrentProc := ModelDef.numstate + 34;
With proc[CurrentProc] do
   begin
      name       := 'Resp: tot metabolic woo';
      units       := 'g C m-2 day-1';
      symbol       := 'RWmet';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Wood resp corr factor';  units:='unitless';  symbol:='awr';
 end;
with par[npar + 2] do
 begin
    name:='Min. active Wood C/N';  units:='g C g-1 N';  symbol:='qWmin';
 end;
 
CurrentProc := ModelDef.numstate + 35;
With proc[CurrentProc] do
   begin
      name       := 'Resp: tot metabolic roo';
      units       := 'g C m-2 day-1';
      symbol       := 'RRmet';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Min. active Root C/N';  units:='g C g-1 N';  symbol:='qRmin';
 end;
 
CurrentProc := ModelDef.numstate + 36;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RFgr';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='growth resp coeff';  units:='fraction';  symbol:='rg';
 end;
 
CurrentProc := ModelDef.numstate + 37;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RWgr';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 38;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth root';
      units       := 'g C m-2 day-1';
      symbol       := 'RRgr';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 39;
With proc[CurrentProc] do
   begin
      name       := 'Labile Contributing Fra';
      units       := 'unitless';
      symbol       := 'fL';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='rate of Cl cutoff';  units:='m2 g-1 C';  symbol:='sL';
 end;
 
CurrentProc := ModelDef.numstate + 40;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RfmetL';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 41;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RwmetL';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 42;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Root';
      units       := 'g C m-2 day-1';
      symbol       := 'RrmetL';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 43;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RfmetS';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 44;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RwmetS';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 45;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Root';
      units       := 'g C m-2 day-1';
      symbol       := 'RrmetS';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 46;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp Fol';
      units       := 'g N m-2 day-1';
      symbol       := 'tauNRf';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 47;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp woo';
      units       := 'g N m-2 day-1';
      symbol       := 'tauNRsw';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 48;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp roo';
      units       := 'g N m-2 day-1';
      symbol       := 'tauNRr';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 49;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Phenology';
      units       := 'unitless';
      symbol       := 'Pf';
      parameters       := 6;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Day to begin DD count';  units:='Julian day';  symbol:='Jddstart';
 end;
with par[npar + 2] do
 begin
    name:='Degree day fol start';  units:='oC day';  symbol:='DDfstart';
 end;
with par[npar + 3] do
 begin
    name:='full canopy degree day';  units:='oC day';  symbol:='DDfstop';
 end;
with par[npar + 4] do
 begin
    name:='Earliest Litter Start';  units:='Julian Day';  symbol:='JlambdaStart';
 end;
with par[npar + 5] do
 begin
    name:='Crit Temp for Fol fall';  units:='oC';  symbol:='TacrF';
 end;
with par[npar + 6] do
 begin
    name:='Duration fol senes';  units:='days';  symbol:='DsensF';
 end;
 
CurrentProc := ModelDef.numstate + 50;
With proc[CurrentProc] do
   begin
      name       := 'Wood Phenology';
      units       := 'unitless';
      symbol       := 'Pw';
      parameters       := 4;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Degree day wood start';  units:='oC day';  symbol:='DDwstart';
 end;
with par[npar + 2] do
 begin
    name:='Degree day wood stop';  units:='oC day';  symbol:='DDwstop';
 end;
with par[npar + 3] do
 begin
    name:='Crit Temp for Wood fall';  units:='oC';  symbol:='TacrW';
 end;
with par[npar + 4] do
 begin
    name:='Duration wood senes';  units:='days';  symbol:='DsensW';
 end;
 
CurrentProc := ModelDef.numstate + 51;
With proc[CurrentProc] do
   begin
      name       := 'Root Phenology';
      units       := 'unitless';
      symbol       := 'Pr';
      parameters       := 4;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Degree day root start';  units:='oC day';  symbol:='DDrstart';
 end;
with par[npar + 2] do
 begin
    name:='Degree day root stop';  units:='oC day';  symbol:='DDrstop';
 end;
with par[npar + 3] do
 begin
    name:='Crit Temp for Root fall';  units:='oC';  symbol:='TacrR';
 end;
with par[npar + 4] do
 begin
    name:='Duration Root senes';  units:='days';  symbol:='DsensR';
 end;
 
CurrentProc := ModelDef.numstate + 52;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Growing Season';
      units       := 'unitless';
      symbol       := 'FGrow';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 53;
With proc[CurrentProc] do
   begin
      name       := 'Wood Growing Season';
      units       := 'unitless';
      symbol       := 'WGrow';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 54;
With proc[CurrentProc] do
   begin
      name       := 'Root Growing Season';
      units       := 'unitless';
      symbol       := 'RGrow';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 55;
With proc[CurrentProc] do
   begin
      name       := 'Allocation: Foliage Fra';
      units       := 'unitless';
      symbol       := 'Af';
      parameters       := 2;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Optimum labile C:N';  units:='g C g-1 N';  symbol:='qLopt';
 end;
with par[npar + 2] do
 begin
    name:='allocation responsiveness';  units:='unitless';  symbol:='thetaG';
 end;
 
CurrentProc := ModelDef.numstate + 56;
With proc[CurrentProc] do
   begin
      name       := 'Allocation: Root Frac';
      units       := 'unitless';
      symbol       := 'Ar';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 57;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Foliage C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcf';
      parameters       := 3;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Foliage growth rate';  units:='day-1';  symbol:='gf';
 end;
with par[npar + 2] do
 begin
    name:='Fol. N growth parameter';  units:='unitless';  symbol:='bGf';
 end;
with par[npar + 3] do
 begin
    name:='Fol. growth sensitivity';  units:='unitless';  symbol:='epsilonF';
 end;
 
CurrentProc := ModelDef.numstate + 58;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Sapwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcsw';
      parameters       := 3;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Sapwood growth rate';  units:='day-1';  symbol:='gSW';
 end;
with par[npar + 2] do
 begin
    name:='Sapwood N growth par';  units:='unitless';  symbol:='bGsw';
 end;
with par[npar + 3] do
 begin
    name:='Sapwood growth sensitivity';  units:='unitless';  symbol:='epsilonW';
 end;
 
CurrentProc := ModelDef.numstate + 59;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Heartwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gchw';
      parameters       := 1;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Heartwood growth rate';  units:='day-1';  symbol:='gHW';
 end;
 
CurrentProc := ModelDef.numstate + 60;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Root C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcr';
      parameters       := 3;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Root growth rate';  units:='day-1';  symbol:='gr';
 end;
with par[npar + 2] do
 begin
    name:='Root N growth parameter';  units:='unitless';  symbol:='bGr';
 end;
with par[npar + 3] do
 begin
    name:='Root growth sensitivity';  units:='unitless';  symbol:='epsilonR';
 end;
 
CurrentProc := ModelDef.numstate + 61;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Foliage N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnf';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 62;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Sapwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnsw';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 63;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Heartwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnhw';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 64;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Root N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnr';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 65;
With proc[CurrentProc] do
   begin
      name       := 'Litter: drought Fol C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCFD';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='fol drought litter rate';  units:='day-1';  symbol:='alambdaFD';
 end;
with par[npar + 2] do
 begin
    name:='root drought litter rate';  units:='day-1';  symbol:='alambdaRD';
 end;
with par[npar + 3] do
 begin
    name:='Drought avail H2O';  units:='fraction';  symbol:='Awd';
 end;
 
CurrentProc := ModelDef.numstate + 66;
With proc[CurrentProc] do
   begin
      name       := 'Litter: drought Root C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCRD';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 67;
With proc[CurrentProc] do
   begin
      name       := 'Litter: drought Fol N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNFD';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 68;
With proc[CurrentProc] do
   begin
      name       := 'Litter: drought Root N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNRD';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 69;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Litter Season';
      units       := 'unitless';
      symbol       := 'FFall';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 70;
With proc[CurrentProc] do
   begin
      name       := 'Wood Litter Season';
      units       := 'unitless';
      symbol       := 'WFall';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 71;
With proc[CurrentProc] do
   begin
      name       := 'Root Litter Season';
      units       := 'unitless';
      symbol       := 'RFall';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 72;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Foliage C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCFs';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='F litter rate';  units:='day-1';  symbol:='alambdaF';
 end;
with par[npar + 2] do
 begin
    name:='F Litter exp';  units:='unitless';  symbol:='sigmaLambdaF';
 end;
with par[npar + 3] do
 begin
    name:='Min. overwinter foliage';  units:='g C m-2';  symbol:='Cfo';
 end;
 
CurrentProc := ModelDef.numstate + 73;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Sapwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCSWs';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='W litter rate';  units:='day-1';  symbol:='alambdaW';
 end;
with par[npar + 2] do
 begin
    name:='W litter exp';  units:='unitless';  symbol:='sigmaLambdaW';
 end;
with par[npar + 3] do
 begin
    name:='Min. overwinter wood';  units:='g C m-2';  symbol:='Cwo';
 end;
 
CurrentProc := ModelDef.numstate + 74;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Heartwood ';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCHWs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 75;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Root C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCRs';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='R litter rate';  units:='day-1';  symbol:='alambdaR';
 end;
with par[npar + 2] do
 begin
    name:='R litter exp';  units:='unitless';  symbol:='sigmaLambdaR';
 end;
with par[npar + 3] do
 begin
    name:='Min. overwinter root';  units:='g C m-2';  symbol:='Cro';
 end;
 
CurrentProc := ModelDef.numstate + 76;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Foliage N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNFs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 77;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Sapwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNSWs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 78;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Heartwood ';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNHWs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 79;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sens Root N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNRs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 80;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total fol C dly';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCF';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 81;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total SW C dly';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCSW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 82;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total HW C dly';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCHW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 83;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total root C dl';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCR';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 84;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total fol N dly';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNF';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 85;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total SW N dly';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNSW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 86;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total HW N dly';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNHW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 87;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total root N dl';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNR';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 88;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total wood C dl';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 89;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Total wood N dl';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 90;
With proc[CurrentProc] do
   begin
      name       := 'C input to extractives';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaE';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 91;
With proc[CurrentProc] do
   begin
      name       := 'C input to acid soluble';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaS';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 92;
With proc[CurrentProc] do
   begin
      name       := 'C input to acid insolub';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaI';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 93;
With proc[CurrentProc] do
   begin
      name       := 'N input to active N';
      units       := 'g N m-2 day-1';
      symbol       := 'FNlambdaA';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 94;
With proc[CurrentProc] do
   begin
      name       := 'Soil metabolic rate';
      units       := 'unitless';
      symbol       := 'rSm';
      parameters       := 4;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Q10 for decomp';  units:='unitless';  symbol:='Q10XI';
 end;
with par[npar + 2] do
 begin
    name:='Decomp H2O skewness';  units:='unitless';  symbol:='sigmaXi';
 end;
with par[npar + 3] do
 begin
    name:='Decomp opt available H2O';  units:='unitless';  symbol:='Awxiopt';
 end;
with par[npar + 4] do
 begin
    name:='Decomp H2O breadth';  units:='unitless';  symbol:='Jxib';
 end;
 
CurrentProc := ModelDef.numstate + 95;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Sol-Extr flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCSE';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Fcse rate const';  units:='day-1';  symbol:='aSE';
 end;
 
CurrentProc := ModelDef.numstate + 96;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Extr-Insol flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCEI';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Fcei rate const';  units:='day-1';  symbol:='aEI';
 end;
 
CurrentProc := ModelDef.numstate + 97;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Insol-Extr flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCIE';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Fcie rate const';  units:='day-1';  symbol:='aIE';
 end;
 
CurrentProc := ModelDef.numstate + 98;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Insol-Humus flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCIH';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Fcih rate const';  units:='day-1';  symbol:='aIH';
 end;
 
CurrentProc := ModelDef.numstate + 99;
With proc[CurrentProc] do
   begin
      name       := 'Soil resp active';
      units       := 'g C m-2 day-1';
      symbol       := 'RE';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='resp efficiency';  units:='fraction';  symbol:='beta';
 end;
 
CurrentProc := ModelDef.numstate + 100;
With proc[CurrentProc] do
   begin
      name       := 'Soil resp humus';
      units       := 'g C m-2 day-1';
      symbol       := 'RH';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Humus resp rate';  units:='day-1';  symbol:='aH';
 end;
 
CurrentProc := ModelDef.numstate + 101;
With proc[CurrentProc] do
   begin
      name       := 'N: gross active min dai';
      units       := 'g N m-2 day-1';
      symbol       := 'MNA';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Na mineralization rate';  units:='day-1';  symbol:='aA';
 end;
 
CurrentProc := ModelDef.numstate + 102;
With proc[CurrentProc] do
   begin
      name       := 'N: gross humus min dail';
      units       := 'g N m-2 day-1';
      symbol       := 'MNH';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 103;
With proc[CurrentProc] do
   begin
      name       := 'Soil N act-humus flux';
      units       := 'g N m-2 day-1';
      symbol       := 'FNAH';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='opt humus C:N ratio';  units:='g C g-1 N';  symbol:='qShopt';
 end;
 
CurrentProc := ModelDef.numstate + 104;
With proc[CurrentProc] do
   begin
      name       := 'NH4: immobilization';
      units       := 'g N m-2 day-1';
      symbol       := 'INH4';
      parameters       := 2;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max NH4 imm rate';  units:='day-1';  symbol:='VINH4max';
 end;
with par[npar + 2] do
 begin
    name:='1/2 sat for NH4 imm';  units:='umol N L-1 H2O';  symbol:='kINH4';
 end;
 
CurrentProc := ModelDef.numstate + 105;
With proc[CurrentProc] do
   begin
      name       := 'NO3: immobilization';
      units       := 'g N m-2 day-1';
      symbol       := 'INO3';
      parameters       := 2;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max NO3 imm rate';  units:='day-1';  symbol:='VINO3max';
 end;
with par[npar + 2] do
 begin
    name:='1/2 sat for NO3 imm';  units:='umol N L-1 H2O';  symbol:='kINO3';
 end;
 
CurrentProc := ModelDef.numstate + 106;
With proc[CurrentProc] do
   begin
      name       := 'Nitrification';
      units       := 'g N m-2 day-1';
      symbol       := 'Fntri';
      parameters       := 1;
      ptype       := ptGroup3;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='nitr. rate';  units:='gN LH2O umol-1N m-2 day-1';  symbol:='aNtri';
 end;
 
CurrentProc := ModelDef.numstate + 107;
With proc[CurrentProc] do
   begin
      name       := 'C: Total Wood';
      units       := 'g C m-2 ground';
      symbol       := 'Cw';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 108;
With proc[CurrentProc] do
   begin
      name       := 'N: Total Wood';
      units       := 'g N m-2 ground';
      symbol       := 'Nw';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 109;
With proc[CurrentProc] do
   begin
      name       := 'C:N Foliage';
      units       := 'g C g-1 N';
      symbol       := 'qF';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 110;
With proc[CurrentProc] do
   begin
      name       := 'C:N Sapwood';
      units       := 'g C g-1 N';
      symbol       := 'qSW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 111;
With proc[CurrentProc] do
   begin
      name       := 'C:N Heartwood';
      units       := 'g C g-1 N';
      symbol       := 'qHW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 112;
With proc[CurrentProc] do
   begin
      name       := 'C:N Total wood';
      units       := 'g C g-1 N';
      symbol       := 'qW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 113;
With proc[CurrentProc] do
   begin
      name       := 'C:N Root';
      units       := 'g C g-1 N';
      symbol       := 'qR';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 114;
With proc[CurrentProc] do
   begin
      name       := 'C:N Labile';
      units       := 'g C g-1 N';
      symbol       := 'qL';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 115;
With proc[CurrentProc] do
   begin
      name       := 'C:N Soil Ci/Na';
      units       := 'g C g-1 N';
      symbol       := 'qSia';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 116;
With proc[CurrentProc] do
   begin
      name       := 'C:N Soil Humus';
      units       := 'g C g-1 N';
      symbol       := 'qSh';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 117;
With proc[CurrentProc] do
   begin
      name       := 'Root:Shoot ratio';
      units       := 'unitless';
      symbol       := 'qrs';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 118;
With proc[CurrentProc] do
   begin
      name       := 'C: total plant';
      units       := 'g C m-2 ground';
      symbol       := 'CTP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 119;
With proc[CurrentProc] do
   begin
      name       := 'C: total active soil';
      units       := 'g C m-2 ground';
      symbol       := 'CTSa';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 120;
With proc[CurrentProc] do
   begin
      name       := 'C: total organic soil';
      units       := 'g C m-2 ground';
      symbol       := 'CTS';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 121;
With proc[CurrentProc] do
   begin
      name       := 'N: total plant';
      units       := 'g N m-2 ground';
      symbol       := 'NTP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 122;
With proc[CurrentProc] do
   begin
      name       := 'N: total organic soil';
      units       := 'g N m-2 ground';
      symbol       := 'NTS';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 123;
With proc[CurrentProc] do
   begin
      name       := 'C:N total plant';
      units       := 'g C g-1 N';
      symbol       := 'qTP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 124;
With proc[CurrentProc] do
   begin
      name       := 'C:N total soil';
      units       := 'g C g-1 N';
      symbol       := 'qTS';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 125;
With proc[CurrentProc] do
   begin
      name       := 'N: total inorganic soil';
      units       := 'g N m-2 ground';
      symbol       := 'NTI';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 126;
With proc[CurrentProc] do
   begin
      name       := 'C: total ecosystem';
      units       := 'g C m-2 ground';
      symbol       := 'CTE';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 127;
With proc[CurrentProc] do
   begin
      name       := 'N: total ecosystem';
      units       := 'g N m-2 ground';
      symbol       := 'NTE';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 128;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total foliage dai';
      units       := 'g C m-2 day-1';
      symbol       := 'RF';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 129;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total wood daily';
      units       := 'g C m-2 day-1';
      symbol       := 'RW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 130;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total root daily';
      units       := 'g C m-2 day-1';
      symbol       := 'RR';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 131;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total plant daily';
      units       := 'g C m-2 day-1';
      symbol       := 'Rp';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 132;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total soil daily';
      units       := 'g C m-2 day-1';
      symbol       := 'Rs';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 133;
With proc[CurrentProc] do
   begin
      name       := 'NPP: foliar daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPf';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 134;
With proc[CurrentProc] do
   begin
      name       := 'NPP: wood daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPw';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 135;
With proc[CurrentProc] do
   begin
      name       := 'NPP: root daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPr';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 136;
With proc[CurrentProc] do
   begin
      name       := 'NPP: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NPP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 137;
With proc[CurrentProc] do
   begin
      name       := 'NEP: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NEP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 138;
With proc[CurrentProc] do
   begin
      name       := 'Litter: daily total C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 139;
With proc[CurrentProc] do
   begin
      name       := 'Litter: daily total N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 140;
With proc[CurrentProc] do
   begin
      name       := 'N: Total daily depos.';
      units       := 'g N m-2 day-1';
      symbol       := 'Dn';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 141;
With proc[CurrentProc] do
   begin
      name       := 'N: Total translocation ';
      units       := 'g N m-2 day-1';
      symbol       := 'tauN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 142;
With proc[CurrentProc] do
   begin
      name       := 'N: Inorganic uptake dly';
      units       := 'g N m-2 day-1';
      symbol       := 'Uni';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 143;
With proc[CurrentProc] do
   begin
      name       := 'N: Uptake daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Un';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 144;
With proc[CurrentProc] do
   begin
      name       := 'N: Gross min total dail';
      units       := 'g N m-2 day-1';
      symbol       := 'Mn';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 145;
With proc[CurrentProc] do
   begin
      name       := 'N: Immob total daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Inn';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 146;
With proc[CurrentProc] do
   begin
      name       := 'N: Total net min daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Nmin';
      parameters       := 0;
      ptype       := ptGroup3;
   end;
 
CurrentProc := ModelDef.numstate + 147;
With proc[CurrentProc] do
   begin
      name       := 'N: Leaching total dly';
      units       := 'g N m-2 day-1';
      symbol       := 'Qn';
      parameters       := 0;
      ptype       := ptGroup3;
   end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 18;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='CO2 Atm';  units:='ml m-3';  symbol:='Ca';
 end;
 
with drive[2] do
 begin
    name:='Temp. Air Max daily';  units:='oC';  symbol:='Tamax';
 end;
 
with drive[3] do
 begin
    name:='Temp. Air Min daily';  units:='oC';  symbol:='Tamin';
 end;
 
with drive[4] do
 begin
    name:='Temp. Soil mean daily';  units:='oC';  symbol:='Ts';
 end;
 
with drive[5] do
 begin
    name:='H2O: precip';  units:='mm H2O day-1';  symbol:='Ppt';
 end;
 
with drive[6] do
 begin
    name:='Incid Shortwave';  units:='MJ m-2 day-1';  symbol:='II';
 end;
 
with drive[7] do
 begin
    name:='NH4: deposition';  units:='g N m-2 day-1';  symbol:='DNH4';
 end;
 
with drive[8] do
 begin
    name:='NO3: deposition';  units:='g N m-2 day-1';  symbol:='DNO3';
 end;
 
with drive[9] do
 begin
    name:='Driver: Fol Litter C';  units:='g C m-2 day-1';  symbol:='DlambdaCF';
 end;
 
with drive[10] do
 begin
    name:='Driver: Sapwood Litter ';  units:='g C m-2 day-1';  symbol:='DlambdaCSW';
 end;
 
with drive[11] do
 begin
    name:='Driver: Heartwood Litte';  units:='g C m-2 day-1';  symbol:='DlambdaCHW';
 end;
 
with drive[12] do
 begin
    name:='Driver: Root Litter C';  units:='g C m-2 day-1';  symbol:='DlambdaCR';
 end;
 
with drive[13] do
 begin
    name:='Driver: Fol Litter N';  units:='g N m-2 day-1';  symbol:='DlambdaNF';
 end;
 
with drive[14] do
 begin
    name:='Driver: Sapwood Litter ';  units:='g N m-2 day-1';  symbol:='DlambdaNSW';
 end;
 
with drive[15] do
 begin
    name:='Driver: Heartwood Litte';  units:='g N m-2 day-1';  symbol:='DlambdaNHW';
 end;
 
with drive[16] do
 begin
    name:='Driver: Root Litter N';  units:='g N m-2 day-1';  symbol:='DlambdaNR';
 end;
 
with drive[17] do
 begin
    name:='Driver: Soil NH4';  units:='g N m-2';  symbol:='DNnh4';
 end;
 
with drive[18] do
 begin
    name:='Driver: Soil NO3';  units:='g N m-2';  symbol:='DNno3';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

// Choose either GlobalPs, ArcticPs, or None here so the appropriate Ps model is compiled below.
{$DEFINE ArcticPs}

PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
{$IFDEF GlobalPs}
const
// Global Ps parameters
 x1 = 11.04;             x2 = 0.03;
 x5 = 0.216;             x6 = 0.6;
 x7 = 3.332;             x8 = 0.004;
 x9 = 1.549;             x10 = 1.156;
 gammastar = 0;          kCO2 = 995.4;  }
{$ENDIF}

// Modify constant above (line above "procedure processes..." line )to specify
// which Ps model and it's constants should be compiled. Choosing a Ps model
// automatically includes the Et and Misc constants (i.e. Gem is assumed).

{$IFDEF ArcticPs}
const
// Arctic Ps parameters
x1 = 0.192;	x2 = 0.125;
x5 = 2.196;	x6 = 50.41;
x7 = 0.161;	x8 = 14.78;
x9 = 1.146;
gammastar = 0.468;	kCO2 = 500.3;
{$ENDIF}

{$IFDEF ArcticPs OR GlobalPs}
//const
// General Et parameters
aE1 = 0.0004;    aE2 = 150;  aE3 = 1.21;   aE4 = 6.11262E5;

// Other constants
cp = 1.012E-9; //specific heat air MJ kg-1 oC-1
sigmaSB = 4.9e-9; //stefan-Boltzmann MJ m-2 day-1 K-4
S0 = 117.5; //solar constant MJ m-2 day-1
bHI1 =0.23;
bHI2 =0.48;
mw = 2.99; //kg h2o MJ-1
alphaMS = 2; //mm oC-1 day-1                                 }
{$ENDIF}

var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
Cf, dCfdt, 
Csw, dCswdt, 
Chw, dChwdt, 
Cr, dCrdt, 
Nf, dNfdt, 
Nsw, dNswdt, 
Nhw, dNhwdt, 
Nr, dNrdt, 
Cl, dCldt, 
Nl, dNldt, 
Cs, dCsdt, 
Ce, dCedt, 
Ci, dCidt, 
Ch, dChdt, 
Na, dNadt, 
Nh, dNhdt, 
Nnh4, dNnh4dt, 
Nno3, dNno3dt, 
Wr, dWrdt, 
Wg, dWgdt, 
Ws, dWsdt, 
Laieos, dLaieosdt, 
Jday, dJdaydt, 
DDa, dDDadt, 
DDs, dDDsdt, 
qUnT, dqUnTdt, 
PptT, dPptTdt, 
PptIT, dPptITdt, 
EtT, dEtTdt, 
QgT, dQgTdt, 
QwT, dQwTdt, 
DnT, dDnTdt, 
PsT, dPsTdt, 
NPPT, dNPPTdt, 
NPPFT, dNPPFTdt, 
NPPWT, dNPPWTdt, 
NPPRT, dNPPRTdt, 
tauNT, dtauNTdt, 
RPT, dRPTdt, 
RfT, dRfTdt, 
RwT, dRwTdt, 
RrT, dRrTdt, 
lambdaCfT, dlambdaCfTdt, 
lambdaCwT, dlambdaCwTdt, 
lambdaCrT, dlambdaCrTdt, 
lambdaNfT, dlambdaNfTdt, 
lambdaNwT, dlambdaNwTdt, 
lambdaNrT, dlambdaNrTdt, 
lambdaCT, dlambdaCTdt, 
lambdaNT, dlambdaNTdt, 
RST, dRSTdt, 
NepT, dNepTdt, 
UNT, dUNTdt, 
UnfixT, dUnfixTdt, 
MnT, dMnTdt, 
InnT, dInnTdt, 
NminT, dNminTdt, 
QnT, dQnTdt, 
Jdfs, dJdfsdt, 
Jdws, dJdwsdt, 
Jdrs, dJdrsdt :double;

{processes and associated parameters}
Ta, 
Hd, Deltad, 
Pptr, Tas, 
Ppts, 
PptI, bIf, bIsw, 
Aw, Womega, Wf, 
PsiD, PsiOmega, Psif, 
gcW, 
ET, alphaT, gAW, 
Ms, rhot, alphas, gammat, 
DR, alphaD, 
Qg, alphaQ, 
Nnh4aq, rhoS, z, Vsnh4max, etaNH4, 
Nno3aq, Vsno3max, etaNO3, 
Nnh4s, 
Nno3s, 
QNH4, 
QNO3, 
Cswreq, bcsw1, bcsw2, 
gcC, omegaS, 
LAI, mF, 
Ps, aNf, Clmid, epsilonPs, 
deltaPs, deltaCa, aDelta, bDelta, 
tauND, atauD, 
tauNlambda, atau, 
HU, TUmax, TUopt, gammaU, 
UNH4, UNH4max, kNH4, thetaU, Nlmid, epsilonUn, 
UNO3, UNO3max, kNO3, 
qUn, gUn, 
Unfix, qUnfix, gNfix, kNfix, 
RUnfix, 
HR, TRmax, TRopt, gammaR, 
RFmet, rm, qFmin, 
RWmet, awr, qWmin, 
RRmet, qRmin, 
RFgr, rg, 
RWgr, 
RRgr, 
fL, sL, 
RfmetL, 
RwmetL, 
RrmetL, 
RfmetS, 
RwmetS, 
RrmetS, 
tauNRf, 
tauNRsw, 
tauNRr, 
Pf, Jddstart, DDfstart, DDfstop, JlambdaStart, TacrF, DsensF, 
Pw, DDwstart, DDwstop, TacrW, DsensW, 
Pr, DDrstart, DDrstop, TacrR, DsensR, 
FGrow, 
WGrow, 
RGrow, 
Af, qLopt, thetaG, 
Ar, 
Gcf, gf, bGf, epsilonF, 
Gcsw, gSW, bGsw, epsilonW, 
Gchw, gHW, 
Gcr, gr, bGr, epsilonR, 
Gnf, 
Gnsw, 
Gnhw, 
Gnr, 
lambdaCFD, alambdaFD, alambdaRD, Awd, 
lambdaCRD, 
lambdaNFD, 
lambdaNRD, 
FFall, 
WFall, 
RFall, 
lambdaCFs, alambdaF, sigmaLambdaF, Cfo, 
lambdaCSWs, alambdaW, sigmaLambdaW, Cwo, 
lambdaCHWs, 
lambdaCRs, alambdaR, sigmaLambdaR, Cro, 
lambdaNFs, 
lambdaNSWs, 
lambdaNHWs, 
lambdaNRs, 
lambdaCF, 
lambdaCSW, 
lambdaCHW, 
lambdaCR, 
lambdaNF, 
lambdaNSW, 
lambdaNHW, 
lambdaNR, 
lambdaCW, 
lambdaNW, 
FClambdaE, 
FClambdaS, 
FClambdaI, 
FNlambdaA, 
rSm, Q10XI, sigmaXi, Awxiopt, Jxib, 
FCSE, aSE, 
FCEI, aEI, 
FCIE, aIE, 
FCIH, aIH, 
RE, beta, 
RH, aH, 
MNA, aA, 
MNH, 
FNAH, qShopt, 
INH4, VINH4max, kINH4, 
INO3, VINO3max, kINO3, 
Fntri, aNtri, 
Cw, 
Nw, 
qF, 
qSW, 
qHW, 
qW, 
qR, 
qL, 
qSia, 
qSh, 
qrs, 
CTP, 
CTSa, 
CTS, 
NTP, 
NTS, 
qTP, 
qTS, 
NTI, 
CTE, 
NTE, 
RF, 
RW, 
RR, 
Rp, 
Rs, 
NPPf, 
NPPw, 
NPPr, 
NPP, 
NEP, 
lambdaC, 
lambdaN, 
Dn, 
tauN, 
Uni, 
Un, 
Mn, 
Inn, 
Nmin, 
Qn, 

{drivers}
Ca, 
Tamax, 
Tamin, 
Ts, 
Ppt, 
II, 
DNH4, 
DNO3, 
DlambdaCF, 
DlambdaCSW, 
DlambdaCHW, 
DlambdaCR, 
DlambdaNF, 
DlambdaNSW, 
DlambdaNHW, 
DlambdaNR, 
DNnh4, 
DNno3, 

{other}
gamma, 
rhoa, 
Deltaes, 
esd, 
lambdav, 
deltar, 
delta, 
Pn, 
E0, 
Cm, 
Pd, 
PII, 
I0, 
HI, 
Ins, 
Tka, 
Tks, 
Jxi, 
Jdaytemp

:double;
npar, j, tnum:integer;

{ Functions or procedures }

begin
{ Copy the drivers from the global array, drive, into the local variables. }
Ca := tdrive[1].value;
Tamax := tdrive[2].value;
Tamin := tdrive[3].value;
Ts := tdrive[4].value;
Ppt := tdrive[5].value;
II := tdrive[6].value;
DNH4 := tdrive[7].value;
DNO3 := tdrive[8].value;
DlambdaCF := tdrive[9].value;
DlambdaCSW := tdrive[10].value;
DlambdaCHW := tdrive[11].value;
DlambdaCR := tdrive[12].value;
DlambdaNF := tdrive[13].value;
DlambdaNSW := tdrive[14].value;
DlambdaNHW := tdrive[15].value;
DlambdaNR := tdrive[16].value;
DNnh4 := tdrive[17].value;
DNno3 := tdrive[18].value;

{ Copy the state variables from the global array into the local variables. }
Cf := tstat[1].value;
Csw := tstat[2].value;
Chw := tstat[3].value;
Cr := tstat[4].value;
Nf := tstat[5].value;
Nsw := tstat[6].value;
Nhw := tstat[7].value;
Nr := tstat[8].value;
Cl := tstat[9].value;
Nl := tstat[10].value;
Cs := tstat[11].value;
Ce := tstat[12].value;
Ci := tstat[13].value;
Ch := tstat[14].value;
Na := tstat[15].value;
Nh := tstat[16].value;
Nnh4 := tstat[17].value;
Nno3 := tstat[18].value;
Wr := tstat[19].value;
Wg := tstat[20].value;
Ws := tstat[21].value;
Laieos := tstat[22].value;
Jday := tstat[23].value;
DDa := tstat[24].value;
DDs := tstat[25].value;
qUnT := tstat[26].value;
PptT := tstat[27].value;
PptIT := tstat[28].value;
EtT := tstat[29].value;
QgT := tstat[30].value;
QwT := tstat[31].value;
DnT := tstat[32].value;
PsT := tstat[33].value;
NPPT := tstat[34].value;
NPPFT := tstat[35].value;
NPPWT := tstat[36].value;
NPPRT := tstat[37].value;
tauNT := tstat[38].value;
RPT := tstat[39].value;
RfT := tstat[40].value;
RwT := tstat[41].value;
RrT := tstat[42].value;
lambdaCfT := tstat[43].value;
lambdaCwT := tstat[44].value;
lambdaCrT := tstat[45].value;
lambdaNfT := tstat[46].value;
lambdaNwT := tstat[47].value;
lambdaNrT := tstat[48].value;
lambdaCT := tstat[49].value;
lambdaNT := tstat[50].value;
RST := tstat[51].value;
NepT := tstat[52].value;
UNT := tstat[53].value;
UnfixT := tstat[54].value;
MnT := tstat[55].value;
InnT := tstat[56].value;
NminT := tstat[57].value;
QnT := tstat[58].value;
Jdfs := tstat[59].value;
Jdws := tstat[60].value;
Jdrs := tstat[61].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 2);
Deltad := par[npar + 1].value;

npar:=ParCount(ModelDef.numstate + 3);
Tas := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 5);
bIf := par[npar + 1].value;
bIsw := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 6);
Womega := par[npar + 1].value;
Wf := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 7);
PsiOmega := par[npar + 1].value;
Psif := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 9);
alphaT := par[npar + 1].value;
gAW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 10);
rhot := par[npar + 1].value;
alphas := par[npar + 2].value;
gammat := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 11);
alphaD := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 12);
alphaQ := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 13);
rhoS := par[npar + 1].value;
z := par[npar + 2].value;
Vsnh4max := par[npar + 3].value;
etaNH4 := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 14);
Vsno3max := par[npar + 1].value;
etaNO3 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 19);
bcsw1 := par[npar + 1].value;
bcsw2 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 20);
omegaS := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 21);
mF := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 22);
aNf := par[npar + 1].value;
Clmid := par[npar + 2].value;
epsilonPs := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 23);
deltaCa := par[npar + 1].value;
aDelta := par[npar + 2].value;
bDelta := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 24);
atauD := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 25);
atau := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 26);
TUmax := par[npar + 1].value;
TUopt := par[npar + 2].value;
gammaU := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 27);
UNH4max := par[npar + 1].value;
kNH4 := par[npar + 2].value;
thetaU := par[npar + 3].value;
Nlmid := par[npar + 4].value;
epsilonUn := par[npar + 5].value;
 
npar:=ParCount(ModelDef.numstate + 28);
UNO3max := par[npar + 1].value;
kNO3 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 29);
gUn := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 30);
qUnfix := par[npar + 1].value;
gNfix := par[npar + 2].value;
kNfix := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 32);
TRmax := par[npar + 1].value;
TRopt := par[npar + 2].value;
gammaR := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 33);
rm := par[npar + 1].value;
qFmin := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 34);
awr := par[npar + 1].value;
qWmin := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 35);
qRmin := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 36);
rg := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 39);
sL := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 49);
Jddstart := par[npar + 1].value;
DDfstart := par[npar + 2].value;
DDfstop := par[npar + 3].value;
JlambdaStart := par[npar + 4].value;
TacrF := par[npar + 5].value;
DsensF := par[npar + 6].value;
 
npar:=ParCount(ModelDef.numstate + 50);
DDwstart := par[npar + 1].value;
DDwstop := par[npar + 2].value;
TacrW := par[npar + 3].value;
DsensW := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 51);
DDrstart := par[npar + 1].value;
DDrstop := par[npar + 2].value;
TacrR := par[npar + 3].value;
DsensR := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 55);
qLopt := par[npar + 1].value;
thetaG := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 57);
gf := par[npar + 1].value;
bGf := par[npar + 2].value;
epsilonF := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 58);
gSW := par[npar + 1].value;
bGsw := par[npar + 2].value;
epsilonW := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 59);
gHW := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 60);
gr := par[npar + 1].value;
bGr := par[npar + 2].value;
epsilonR := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 65);
alambdaFD := par[npar + 1].value;
alambdaRD := par[npar + 2].value;
Awd := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 72);
alambdaF := par[npar + 1].value;
sigmaLambdaF := par[npar + 2].value;
Cfo := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 73);
alambdaW := par[npar + 1].value;
sigmaLambdaW := par[npar + 2].value;
Cwo := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 75);
alambdaR := par[npar + 1].value;
sigmaLambdaR := par[npar + 2].value;
Cro := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 94);
Q10XI := par[npar + 1].value;
sigmaXi := par[npar + 2].value;
Awxiopt := par[npar + 3].value;
Jxib := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 95);
aSE := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 96);
aEI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 97);
aIE := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 98);
aIH := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 99);
beta := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 100);
aH := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 101);
aA := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 103);
qShopt := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 104);
VINH4max := par[npar + 1].value;
kINH4 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 105);
VINO3max := par[npar + 1].value;
kINO3 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 106);
aNtri := par[npar + 1].value;
 
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }

{if StateReset then    // Reset variables
begin
   Jday:= 1;
   DDa := 0;
   DDs := 0;
   tnum := FmCalculate.GetArrayIndex(vtstate,'Jday');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := Jday;
   tnum := FmCalculate.GetArrayIndex(vtstate,'DDa');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := DDa;
   tnum := FmCalculate.GetArrayIndex(vtstate,'DDs');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := DDs;
// reset water totals
   PptT := 0;
   PptIT := 0;
   EtT := 0;
   QgT := 0;
   QwT := 0;
   tnum := FmCalculate.GetArrayIndex(vtstate,'PptT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := PptT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'PptIT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := PptIT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'EtT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := EtT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'QgT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := QgT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'QwT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := QwT;
// reset N deposition, GPP, NPP, plant resp, and retranslocation
   DnT := 0;
   PST := 0;
   NPPT := 0;
   NPPFT := 0;
   NPPWT := 0;
   NPPRT := 0;
   tauNT := 0;
   RPT := 0;
   RfT := 0;
   RwT := 0;
   RrT := 0;
   tnum := FmCalculate.GetArrayIndex(vtstate,'DnT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := DnT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'PsT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := PsT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NPPT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NPPT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NPPFT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NPPFT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NPPWT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NPPWT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NPPRT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NPPRT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'tauNT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := tauNT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'RPT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := RPT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'RfT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := RfT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'RwT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := RwT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'RrT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := RrT;
// reset litter totals
   lambdaCfT := 0;
   lambdaCwT := 0;
   lambdaCrT := 0;
   lambdaNfT := 0;
   lambdaNwT := 0;
   lambdaNrT := 0;
   lambdaCT := 0;
   lambdaNT := 0;
   tnum := FmCalculate.GetArrayIndex(vtstate,'lambdaCfT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaCfT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaCwT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaCwT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaCrT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaCrT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaNfT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaNfT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaNwT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaNwT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaNrT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaNrT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaCT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaCT;
   tnum := FmCalculate.GetArrayIndex(vtstate, 'lambdaNT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := lambdaNT;
// reset soil resp, NEPT, N uptake, n mineralization, immobilization, and n leaching
   RST := 0;
   NepT := 0;
   UNT := 0;
   UnfixT := 0;
   NminT := 0;
   QnT := 0;
   MnT := 0;
   InnT := 0;
   tnum := FmCalculate.GetArrayIndex(vtstate,'RST');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := RST;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NepT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NepT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'UNT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := UNT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'UnfixT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := UnfixT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'NminT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := NminT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'QnT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := QnT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'MnT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := MnT;
   tnum := FmCalculate.GetArrayIndex(vtstate,'InnT');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := InnT;
// reset litter fall start
   tnum := FmCalculate.GetArrayIndex(vtstate,'Jdfs');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := 999;
   tnum := FmCalculate.GetArrayIndex(vtstate,'Jdws');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := 999;
   tnum := FmCalculate.GetArrayIndex(vtstate,'Jdrs');
   if tstat[tnum].HoldConstant = False then tstat[tnum].value := 999;
end;}

Jdaytemp := Jday+1*FmOptions.RunOptions.Time_step;

for j := 1 to 21 do                  // negative states
begin
 if tstat[j].value < 0 then tstat[j].value := 0;
end;

// Initialize discrete processes
Pptr := 0;
PptI := 0;
Ppts := 0;
Ms := 0;

// Temperature
Ta := (Tamax+Tamin)/2;

// Total Wood
Cw := Csw + Chw;
Nw := Nsw + Nhw;

// LAI
LAI := Cf/mF;

// C:N ratios
qF := Cf/Nf;
qSW := Csw/Nsw;
if Nhw <> 0 then qHW := Chw/Nhw else qHW := 0;
qW := (Csw+Chw)/(Nsw+Nhw);
qR := Cr/Nr;
qL := Cl/Nl;
qSia := CI/Na;
qSh := Ch/Nh;

// Available water
if Wr<Womega then 
  Aw := 0
else 
  Aw := (Wr-Womega)/(Wf-Womega);

// H2O tension deficit
if Aw>1 then 
  PsiD:= Psif-PsiOmega 
else 
  PsiD := -PsiOmega*(1-power(Psif/PsiOmega,Aw));
if PsiD <= 0 then PsiD:= 0.001;

// canopy H2O conductance and ET
gamma := 6.46E-5*exp(0.00097*Tamax);
rhoa := 353/(273.2+Tamax);
Deltaes := 2.5029*exp(17.269*Tamax/(237.3+Tamax))/sqr(237.3+Tamax);
esd := 6.1078E-4*(exp(17.269*Tamax/(237.3+Tamax))-exp(17.269*Tamin/(237.3+Tamin)));
lambdav := 2.501-2.37e-3*Tamax;
gcW := aE1*(LAI+0.1)*PsiD*exp(-aE2*esd);
ET := lambdav*(Deltaes+gamma*(1+gAW/gcW));
ET := (aE3*Deltaes*II*(1-alphaT)+86400*aE4*rhoa*cp*esd*gAW)/ET;

// DRAINAGE AND DISCHARGE
if (WR>WF) and (Ts>0) then 
  DR:=alphaD*(WR-WF)
else
  DR:= 0;
if Ts > 0 then 
  Qg := alphaQ*WG
else
  Qg := 0;

// Dissolved N
// aqueous NH4 and NO3
Nnh4aq := (Nnh4-rhoS*z*Vsnh4max)/(1.4e-5*WR)-etaNH4;
Nnh4aq := 0.5*(Nnh4aq+sqrt(sqr(Nnh4aq)+4*etaNH4*Nnh4/(1.4e-5*WR)));
Nno3aq := (Nno3-rhoS*z*Vsno3max)/(1.4e-5*WR)-etaNO3;
Nno3aq := 0.5*(Nno3aq+sqrt(sqr(Nno3aq)+4*etaNO3*Nno3/(1.4e-5*WR)));
// sorbed NH4 and NO3
Nnh4s := Nnh4-1.4e-5*WR*Nnh4aq;
Nno3s := Nno3-1.4e-5*WR*Nno3aq;
// N loss
Qnh4 := 1.4e-5*DR*Nnh4aq;
Qno3 := 1.4e-5*DR*Nno3aq;
Dn := Dnh4 + Dno3;
Qn := Qnh4 + Qno3;

// day length
Deltar := Deltad*pi/180;
delta := 0.1303*pi*cos(2*pi*(trunc(Jdaytemp )-171.25)/365.25);
if tan(Deltar)*tan(delta) < -1 then
  Hd := 0
else if tan(Deltar)*tan(delta) > 1 then
  Hd := 24
else
  Hd := 24*arccos(-tan(Deltar)*tan(delta))/pi;

// daily photosynthesis  
E0:=x7;  { This line is here to raise an error at compile time if the photosynthesis constants aren't defined. The value of E0 will be overwritten below. }

// Global
{$IFDEF GlobalPs}
E0 := x7*sqr(LAI)/(x9+sqr(LAI));
gcC := power(abs(PsiD),x10)/(x6*omegas+0.5*(Tamax-Tamin));
PN := x1*aNf*NF*exp(x8*Tamax)/gcC;
PN:=PN/(1+power(CL/Clmid,epsilonPs));
{$ENDIF}

//Arctic only
{$IFDEF ArcticPs}
E0 := x7*sqr(LAI)/(power(LAI,x9));
gcC := -PsiD/(x6*omegas+0.5*(Tamax-Tamin));
PN := x1*aNf*NF/gcC;
PN := PN*x8*Tamax/(x8+Tamax);
PN:=PN/(1+power(CL/Clmid,epsilonPs));
{$ENDIF}

// Global and Arctic
Cm := Ca+Gammastar-kCO2-PN;
Cm := Cm+sqrt(sqr(Cm)-4*(Ca*(gammastar-kco2)-PN*gammastar));
Cm := Cm/2;
PD := gcC*(Ca-Cm);
PII := Pd*E0*II/(PD+E0*II);
if Tamax > 0 then 
  Ps := (x2*Hd+x5)*PII
else
  Ps := 0;

deltaPs := deltaCa - aDelta-(bDelta - aDelta)*(Cm/Ca);

// Sapwood
Cswreq := bcsw1*LAIeos*power(Cw,bcsw2);

// Growing season
if (DDfstart <= DDa) and (DDa <= DDfstop) then
  FGrow := 1
else
  FGrow := 0;

if (DDwstart <= DDa) and (DDa <= DDwstop) then
  WGrow := 1
else
  WGrow := 0;

if (DDrstart <= DDs) and (DDs <= DDrstop) then
  RGrow := 1
else
  RGrow := 0;

// Growth
Af := 1/(1 + power((Cl/Nl)/qLopt, thetaG));
Ar := 1/(1 + power((Cl/Nl)/qLopt, -thetaG));
if (Cl > 0) and (Nl > 0) then
 begin
   Gcf := FGrow*Af*gF*(Cl/Csw)*power(Nl/Csw,1-epsilonF);
   Gnf := FGrow*Af*(gF/bgf)*(Nl/Csw)*power(Cl/Csw,1-epsilonF);
   Gcsw := WGrow*gSW*(Cl/Csw)*power(Nl/Csw,1-epsilonW);
   Gnsw := WGrow*(gSW/bgsw)*(Nl/Csw)*power(Cl/Csw,1-epsilonW);
   Gcr := RGrow*Ar*gr*(Cl/Csw)*power(Nl/Csw,1-epsilonR);
   Gnr := RGrow*Ar*(gr/bgr)*power(Cl/Csw,1-epsilonR)*(Nl/Csw);
 end
else
 begin
   Gcf :=0;
   Gnf :=0;
   Gcsw :=0;
   Gnsw :=0;
   Gcr := 0;
   Gnr := 0;
 end;

if Csw >= Cswreq then
  Gchw := WGrow*gHW*(Csw-Cswreq)
else
  Gchw := 0;
Gnhw := Gchw*Nsw/Csw;

// Respiration
RFgr := rg*GCF;
RWgr := rg*GCSW;
RRgr := rg*GCR;

if Ta<TRmax then
   begin
      HR := (TRmax-Ta)/(Trmax-Tropt);
      HR := power(HR,gammaR*(TRmax-TRopt));
      HR := HR*exp(gammaR*(Ta-TRopt));
   end
else HR:= 0;

if CF/NF>qFmin then 
  RFmet := rm*HR*NF 
else 
  RFmet := rm*HR*CF/qFmin;
if CsW/NsW>qWmin then 
  RWmet := rm*aWR*HR*NSW 
else 
  RWmet := rm*aWR*HR*CSW/qWmin;

if Ts<TRmax then
   begin
      HR := (TRmax-Ts)/(Trmax-Tropt);
      HR := power(HR,gammaR*(TRmax-TRopt));
      HR := HR*exp(gammaR*(Ts-TRopt));
   end
else HR:= 0;

if CR/NR>qRmin then 
  RRmet := rm*HR*NR 
else 
  RRmet := rm*HR*CR/qRmin;

fL := 1-exp(-sL*Cl);
RfmetL := Rfmet*fL;
RwmetL := Rwmet*fL;
RrmetL := Rrmet*fL;
RfmetS := Rfmet*(1-fL);
RwmetS := Rwmet*(1-fL);
RrmetS := Rrmet*(1-fL);

RF := RFmet + RFgr;
RW := RWmet + RWgr;
RR := RRmet + RRgr;

tauNRF := RfmetS/qF;
tauNrsw := RwmetS/qSW;
tauNrr := RrmetS/qR;

// N uptake
if Ts<Tumax then
   begin
      HU := (Tumax-Ts)/(Tumax-Tuopt);
      HU := power(HU,gammaU*(Tumax-Tuopt));
      HU := HU*exp(gammaU*(Ts-Tuopt));
   end
else 
      HU:= 0;

Unh4 := HU*Aw*Unh4max*(1-exp(-thetaU*CR))*Nnh4aq/((kNH4+Nnh4aq));
UNH4:=UNH4/(1+power(NL/Nlmid,epsilonUn));

Uno3 := HU*Aw*Uno3max*(1-exp(-thetaU*CR))*Nno3aq/((kNO3+Nno3aq));
UNo3:=UNo3/(1+power(NL/Nlmid,epsilonUn));

Uni := Unh4 + Uno3;

qUn := gUn*II*exp(thetaU*Cr-0.5*LAI)*Ca/(kCO2+Ca);
qUn := qUn/(Unh4max*NNH4aq/(kNH4 + NNH4aq) + Uno3max*NNO3aq/(kNO3 + NNO3aq));

// N fixation
if qUnT >= qUnfix then
  Unfix := fL*HU*gNfix*Cr*(qUnT-qUnfix)/(knfix+qUnT-qUnfix)
else
  Unfix := 0;
RUnfix := qUnfix*Unfix;

Un := Uni + Unfix;

// Litterfall season
if (Jdaytemp  >= Jdfs) and (Jdaytemp  < Jdfs + Dsensf) then
  FFall := 1
else
  FFall := 0;

if (Jdaytemp  >= Jdws) and (Jdaytemp  < Jdws + Dsensw) then
  WFall := 1
else
  WFall := 0;

if (Jdaytemp  >= Jdrs) and (Jdaytemp  < Jdrs + Dsensr) then
  RFall := 1
else
  RFall := 0;

// Drought Litterfall
if Aw <= Awd then
 begin
   lambdaCFD := alambdaFD*power((Awd - Aw),2)*Cf;
   lambdaCRD := alambdaRD*power((Awd - Aw),2)*Cr;
   lambdaNFD := (1-atauD)*lambdaCFD*Nf/Cf;
   lambdaNRD := lambdaCRD*Nr/Cr;
   tauND := atauD*lambdaCFD*Nf/Cf;
 end
else
 begin
   lambdaCFD := 0;
   lambdaCRD := 0;
   lambdaNFD := 0;
   lambdaNRD := 0;
   tauND := 0;
 end;

// Litterfall
if DlambdaCF = -1 then
 begin
  lambdaCFs := FFall*alambdaF*CF*(1-Cfo/power(power(Cfo,sigmaLambdaF)+
                      power(CF,sigmaLambdaF),1/sigmaLambdaF));
 end
else
 begin
  lambdaCFs := DlambdaCF;
 end;
lambdaCF := lambdaCFs + lambdaCFD;

if DlambdaCSW = -1 then
 begin
  lambdaCSWs := WFall*alambdaW*(Csw/Cw)*Cw*(1-Cwo/power(power(Cwo,sigmaLambdaW) +
                      power(Cw,sigmaLambdaW),1/sigmaLambdaW));
 end
else
 begin
  lambdaCSWs := DlambdaCsw;
 end;
lambdaCSW := lambdaCSWs;

if DlambdaCHW = -1 then
 begin
  lambdaCHWs := WFall*alambdaW*(Chw/Cw)*Cw*(1-Cwo/power(power(Cwo,sigmaLambdaW) +
                      power(Cw,sigmaLambdaW),1/sigmaLambdaW));
 end
else
 begin
  lambdaCHWs := DlambdaCHW;
 end;
lambdaCHW := lambdaCHWs;

if DlambdaCR = -1 then
 begin
  lambdaCRs := RFall*alambdaR*Cr*(1-Cro/power(power(Cro,sigmaLambdaR) +
                      power(Cr,sigmaLambdaR),1/sigmaLambdaR));
 end
else
 begin
  lambdaCRs := DlambdaCR;
 end;
lambdaCR := lambdaCRs + lambdaCRD;

if DlambdaNF = -1 then
 begin
  lambdaNFs := (1-atau)*lambdaCFs*NF/CF;
 end
else
 begin
  lambdaNFs := DlambdaNF;
 end;
lambdaNF := lambdaNFs + lambdaNFD;

if DlambdaNSW = -1 then
 begin
  lambdaNSWs := lambdaCSW*NsW/CsW;
 end
else
 begin
  lambdaNSWs := DlambdaNSW;
 end;
lambdaNSW := lambdaNSWs;

if Chw = 0 then
  lambdaNHWs := 0
else if DlambdaNHW = -1 then
 begin
  lambdaNHWs := lambdaCHW*Nhw/Chw;
 end
else
 begin
  lambdaNHWs := DlambdaNHW;
 end;
lambdaNHW := lambdaNHWs;

if DlambdaNR = -1 then
 begin
  lambdaNRs := lambdaCR*NR/CR;
 end
else
 begin
  lambdaNRs := DlambdaNR;
 end;
lambdaNR := lambdaNRs + lambdaNRD;

// Litterfall totals
lambdaCW := lambdaCSW + lambdaCHW;
lambdaNW := lambdaNSW + lambdaNHW;
lambdaC := lambdaCF + lambdaCW + lambdaCR;
lambdaN := lambdaNF + lambdaNW + lambdaNR;

// Translocation
tauNlambda := atau*lambdaCFs*NF/CF;
tauN := tauNlambda + tauND + tauNrf + tauNrsw + tauNrr;

// Litter partitioning
FClambdaE := 0;
FClambdaS := 0;
if lambdaCF <> 0 then
 begin
   FClambdaE := (0.49 - 0.452*exp(-95*lambdaNF/lambdaCF))*lambdaCF;
   FClambdaS := (0.4 + 0.39*exp(-155*lambdaNF/lambdaCF))*lambdaCF;
 end;
if lambdaCSW <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNSW/lambdaCSW))*lambdaCSW;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNSW/lambdaCSW))*lambdaCSW;
 end;
if lambdaChw <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNhw/lambdaChw))*lambdaChw;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNhw/lambdaChw))*lambdaChw;
 end;
if lambdaCR <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNR/lambdaCR))*lambdaCR;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNR/lambdaCR))*lambdaCR;
 end;
FClambdaI := lambdaC - FClambdaE - FClambdaS;
FNlambdaA := lambdaN;

// Soil metabolic rate
Jxi := 1-sqr(Aw-Awxiopt)/(sqr(Jxib)*(sigmaXi*(Aw-Awxiopt)+1));
if Jxi < 0 then Jxi := 0;
rsm := Jxi*power(Q10xi,Ts/10);

// Soil fluxes
FCSE := rSm*aSE*Cs;
FCEI := rSm*aEI*CE;
FCIE := min(rSm*aIE*CI,rSm*aIE*Na*qShopt);
FCIH := min(rsm*aIH*Ci,rsm*aIH*Na*qShopt);
RE := beta*(FCSE+FCEI+FCIE);
RH := rSm*aH*CH;
MNA := rSm*aA*NA;
MNH := rSm*aH*NH;
FNAH := FCIH/qShopt;
if CI/qShopt>NA then 
 begin
   INH4:=rSm*VINH4max*Nnh4aq*(CI/qShopt - NA)/(kINH4+Nnh4aq);
   INO3:=rSm*VINO3max*Nno3aq*(CI/qShopt - NA)/(kINO3+Nno3aq);
 end
else
 begin
   INH4 := 0;
   INO3 := 0;
 end;
Fntri := rsm*aNtri*Nnh4aq;
Mn := Mna + Mnh;
Inn := Inh4 + Ino3;

// Summaries
Ctp := Cf + Csw + Chw + Cr + Cl;
Ntp := Nf + Nsw + Nhw + Nr + Nl;
Cts := Cs + Ce + Ci + Ch;
Ctsa := Cs + Ce + Ci;
Nts := Na + Nh;
qTP := Ctp/Ntp;
qTS := Cts/Nts;
Nti := Nnh4 + Nno3;
Cte := Ctp + Cts;
Nte := Ntp + Nts + Nti;
qrs := Cr/Cf;
Rp := RF + RW + RR + RUNfix;
Rs := Re + Rh;
NPPF := GCF - RfmetS;
NPPW := GCSW - RwmetS;
NPPR := GCR - RrmetS;
NPP := Ps - Rp;
NEP := NPP - Rs;
Nmin := Mn - Inn;

Pf := 0;     // Just to prevent model from grabbing garbage from memory.
Pw := 0;
Pr := 0;

if CalculateDiscrete then
begin
 // End of year?
 if trunc(time) mod 366 = 0 then
   newyear := true
 else
   newyear := false;

 // increment Julian day
 dJdaydt := 1*FmOptions.RunOptions.DiscreteStep;

 // degree day
 if (Jdaytemp  >= Jddstart) then
  begin
    if Ta > 0 then dDDadt := Ta*FmOptions.RunOptions.DiscreteStep else dDDadt:=0;
    if Ts > 0 then dDDsdt := Ts*FmOptions.RunOptions.DiscreteStep else dDDsdt:=0;
  end
 else
  begin
    dDDadt := 0;
    dDDsdt := 0;
  end;
  
 // laieos
// if DDa = DDfstop then LAIeos := 0;   // fix necessary???  Doesn't seem like it
 if (DDa > DDfstop) and (Jdaytemp  < Jdfs) then
   Laieos := Lai;

 // Litterfall season
 if (Jdaytemp  + dJdaydt >= Jlambdastart) then 
  begin
    tnum := FmCalculate.GetArrayIndex(vtstate,'Jdfs');
    if (Jdfs = 999) and (Ta <= TacrF) then
      tstat[tnum].value := round(Jdaytemp  + dJdaydt);
    tnum := FmCalculate.GetArrayIndex(vtstate,'Jdws');
    if (Jdws = 999) and (Ta <= TacrW) then
      tstat[tnum].value := round(Jdaytemp  + dJdaydt);
    tnum := FmCalculate.GetArrayIndex(vtstate,'Jdrs');
    if (Jdrs = 999) and (Ta <= TacrR) then 
      tstat[tnum].value := round(Jdaytemp  + dJdaydt);  end;

// Cumulative N deposition
 dDnTdt := Dn*FmOptions.RunOptions.DiscreteStep;

 // Rain and Snow
 If Ta<Tas then
   begin
      Pptr := 0;
      PPts := Ppt;
      PptI := 0;
   end
 else
   begin
      Pptr := Ppt;
      PPts := 0;
      PptI := bIsw*CSW + bIf*LAI;
      if PptI > Pptr then PptI := Pptr;
   end;

 // Snowmelt
 I0 := S0*(Hd*sin(Deltar)*sin(delta)/24 + cos(Deltar)*sin(Hd*pi/24)/pi);
 if I0 <> 0 then HI := (II/I0 - bHI1)/bHI2 else HI := 0;
 Tka := Ta + 273.15;
 if Ta>2.5 then Tks := 273.15 else Tks := Tka-2.5;
 Ins := 0.1 + 0.9*HI;
 Ins := Ins*(0.76*power(Tka,4) - power(Tks,4))*(1 - rhot);
 Ins := Ins + rhot*(power(Tka,4) - power(Tks,4));
 Ins := II*exp(-gammat*rhot)*(1 - alphaS) + sigmaSB*Ins;
 MS := 0;
 if Ins > 0 then MS := mw*INS;
 if Ta > 0 then MS := MS + alphaMS*Ta;
 if MS > WS then MS := WS;

 // Water flux derivatives
 dWsdt := (Ppts - Ms)*FmOptions.RunOptions.DiscreteStep;
 dWrdt := (Pptr - PptI + Ms)*FmOptions.RunOptions.DiscreteStep;
 dPptTdt := Ppt*FmOptions.RunOptions.DiscreteStep;
 dPptITdt := PptI*FmOptions.RunOptions.DiscreteStep;
 dQwTdt := dPptITdt;

// Update state variables directly because these are discrete processes and
 // don't pass through the integrator.
 tnum := FmCalculate.GetArrayIndex(vtstate,'Wr');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := Wr + dWrdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'Ws');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := Ws + dWsdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'Laieos');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := Laieos;
 tnum := FmCalculate.GetArrayIndex(vtstate,'Jday');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := Jday + dJdaydt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'DDa');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := DDa + dDDadt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'DDs');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := DDs + dDDsdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'PptT');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := PptT + dPptTdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'PptIT');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := PptIT + dPptITdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'QwT');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := QwT + dQwTdt;
 tnum := FmCalculate.GetArrayIndex(vtstate,'DnT');
 if tstat[tnum].HoldConstant = False then tstat[tnum].value := DnT + dDnTdt;
end; //discrete processes 
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if (tstat[1].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCfdt := 0
else
 dCfdt := Gcf - lambdaCF - Rfmets;
 
if (tstat[2].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCswdt := 0
else
 dCswdt := Gcsw - lambdaCSW - Gchw - Rwmets;
 
if (tstat[3].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dChwdt := 0
else
 dChwdt := Gchw - lambdaChW;
 
if (tstat[4].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCrdt := 0
else
 dCrdt := Gcr - lambdaCR - Rrmets;
 
if (tstat[5].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNfdt := 0
else
 dNfdt := Gnf - lambdaNF - tauNlambda - tauND - tauNRF;
 
if (tstat[6].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNswdt := 0
else
 dNswdt := Gnsw - Gnhw - lambdaNSW - tauNrsw;
 
if (tstat[7].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNhwdt := 0
else
 dNhwdt := Gnhw - lambdaNHW;
 
if (tstat[8].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNrdt := 0
else
 dNrdt := Gnr - lambdaNR - tauNrr;
 
if (tstat[9].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCldt := 0
else
 dCLdt := Ps - Rfgr - RfmetL - Rwgr - RwmetL - Rrgr - RrmetL - RUnfix - Gcf - Gcsw - Gcr;
 
if (tstat[10].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNldt := 0
else
 dNLdt := Uni + Unfix + tauN - Gnf - Gnsw - Gnr;
 
if (tstat[11].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCsdt := 0
else
 dCsdt := FClambdaS - Fcse;
 
if (tstat[12].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCedt := 0
else
 dCedt := FClambdaE + Fcse + Fcie - Re - Fcei;
 
if (tstat[13].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCidt := 0
else
 dCidt := FClambdaI + Fcei - Fcie - Fcih;
 
if (tstat[14].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dChdt := 0
else
 dChdt := Fcih - Rh;
 
if (tstat[15].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNadt := 0
else
 dNadt := FNlambdaA + Inh4 + Ino3 - Fnah - Mna;
 
if (tstat[16].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNhdt := 0
else
 dNhdt := Fnah - Mnh;
 
if (tstat[17].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNnh4dt := 0
else
 dNnh4dt := Dnh4 + Mna + Mnh - Inh4 - Unh4 - Fntri - Qnh4;
 
if (tstat[18].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNno3dt := 0
else
 dNno3dt := Dno3 + Fntri - Uno3 - Ino3 - Qno3;
 
if (tstat[19].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dWrdt := 0
else
 dWRdt := - Et - Dr; // Ms and Pptr are discrete
 
if (tstat[20].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dWgdt := 0
else
 dWGdt := Dr - Qg;
 
if (tstat[21].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dWsdt := 0
else
 dWSdt := 0;   // Function of discrete processes so state variable is updated directly.
 
if (tstat[22].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dLaieosdt := 0
else
 dLaieosdt := 0;  // Function of discrete processes so state variable is updated directly.
 
if (tstat[23].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dJdaydt := 0
else
 dJdaydt := 0;  // Function of discrete processes so state variable is updated directly.
 
if (tstat[24].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDDadt := 0
else
 dDDadt := 0;  // Function of discrete processes so state variable is updated directly.
 
if (tstat[25].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDDsdt := 0
else
 dDDsdt := 0;  // Function of discrete processes so state variable is updated directly.
 
if (tstat[26].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dqUnTdt := 0
else
 dqUnTdt := (qUn-qUnT)/365;
 
if (tstat[27].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dPptTdt := 0
else
 dPptTdt := 0; // Function of discrete processes so state variable is updated directly.
 
if (tstat[28].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dPptITdt := 0
else
 dPptITdt := 0; // Function of discrete processes so state variable is updated directly.
 
if (tstat[29].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dEtTdt := 0
else
 dEtTdt := Et;
 
if (tstat[30].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dQgTdt := 0
else
 dQgTdt := Qg;
 
if (tstat[31].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dQwTdt := 0
else
 dQwTdt := Et + Qg;
 
if (tstat[32].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDnTdt := 0
else
 dDnTdt := 0; 
 
if (tstat[33].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dPsTdt := 0
else
 dPSTdt := Ps;
 
if (tstat[34].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNPPTdt := 0
else
 dNPPTdt := NPP;
 
if (tstat[35].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNPPFTdt := 0
else
 dNPPFTdt := NPPF;
 
if (tstat[36].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNPPWTdt := 0
else
 dNPPWTdt := NPPW;
 
if (tstat[37].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNPPRTdt := 0
else
 dNPPRTdt := NPPR;
 
if (tstat[38].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dtauNTdt := 0
else
 dtauNTdt := tauN;
 
if (tstat[39].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dRPTdt := 0
else
 dRPTdt := Rp;
 
if (tstat[40].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dRfTdt := 0
else
 dRfTdt := Rf;
 
if (tstat[41].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dRwTdt := 0
else
 dRwTdt := Rw;
 
if (tstat[42].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dRrTdt := 0
else
 dRrTdt := Rr;
 
if (tstat[43].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaCfTdt := 0
else
 dlambdaCfTdt := lambdaCf;
 
if (tstat[44].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaCwTdt := 0
else
 dlambdaCwTdt := lambdaCw;
 
if (tstat[45].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaCrTdt := 0
else
 dlambdaCrTdt := lambdaCr;
 
if (tstat[46].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaNfTdt := 0
else
 dlambdaNfTdt := lambdaNf;
 
if (tstat[47].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaNwTdt := 0
else
 dlambdaNwTdt := lambdaNw;
 
if (tstat[48].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaNrTdt := 0
else
 dlambdaNrTdt := lambdaNr;
 
if (tstat[49].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaCTdt := 0
else
 dlambdaCTdt := lambdaC;
 
if (tstat[50].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dlambdaNTdt := 0
else
 dlambdaNTdt := lambdaN;
 
if (tstat[51].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dRSTdt := 0
else
 dRSTdt := Rs;
 
if (tstat[52].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNepTdt := 0
else
 dNEPTdt := NEP;
 
if (tstat[53].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dUNTdt := 0
else
 dUNTdt := Un;
 
if (tstat[54].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dUnfixTdt := 0
else
 dUnfixTdt := Unfix;
 
if (tstat[55].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dMnTdt := 0
else
 dMnTdt := Mn;
 
if (tstat[56].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dInnTdt := 0
else
 dInnTdt := Inn;
 
if (tstat[57].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dNminTdt := 0
else
 dNminTdt := Nmin;
 
if (tstat[58].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dQnTdt := 0
else
 dQnTdt := Qn;
 
if (tstat[59].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dJdfsdt := 0
else
 dJdfsdt := 0;  
 
if (tstat[60].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dJdwsdt := 0
else
 dJdwsdt := 0;  
 
if (tstat[61].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dJdrsdt := 0
else
 dJdrsdt := 0;  
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dCfdt;
tproc[2].value := dCswdt;
tproc[3].value := dChwdt;
tproc[4].value := dCrdt;
tproc[5].value := dNfdt;
tproc[6].value := dNswdt;
tproc[7].value := dNhwdt;
tproc[8].value := dNrdt;
tproc[9].value := dCldt;
tproc[10].value := dNldt;
tproc[11].value := dCsdt;
tproc[12].value := dCedt;
tproc[13].value := dCidt;
tproc[14].value := dChdt;
tproc[15].value := dNadt;
tproc[16].value := dNhdt;
tproc[17].value := dNnh4dt;
tproc[18].value := dNno3dt;
tproc[19].value := dWrdt;
tproc[20].value := dWgdt;
tproc[21].value := dWsdt;
tproc[22].value := dLaieosdt;
tproc[23].value := dJdaydt;
tproc[24].value := dDDadt;
tproc[25].value := dDDsdt;
tproc[26].value := dqUnTdt;
tproc[27].value := dPptTdt;
tproc[28].value := dPptITdt;
tproc[29].value := dEtTdt;
tproc[30].value := dQgTdt;
tproc[31].value := dQwTdt;
tproc[32].value := dDnTdt;
tproc[33].value := dPsTdt;
tproc[34].value := dNPPTdt;
tproc[35].value := dNPPFTdt;
tproc[36].value := dNPPWTdt;
tproc[37].value := dNPPRTdt;
tproc[38].value := dtauNTdt;
tproc[39].value := dRPTdt;
tproc[40].value := dRfTdt;
tproc[41].value := dRwTdt;
tproc[42].value := dRrTdt;
tproc[43].value := dlambdaCfTdt;
tproc[44].value := dlambdaCwTdt;
tproc[45].value := dlambdaCrTdt;
tproc[46].value := dlambdaNfTdt;
tproc[47].value := dlambdaNwTdt;
tproc[48].value := dlambdaNrTdt;
tproc[49].value := dlambdaCTdt;
tproc[50].value := dlambdaNTdt;
tproc[51].value := dRSTdt;
tproc[52].value := dNepTdt;
tproc[53].value := dUNTdt;
tproc[54].value := dUnfixTdt;
tproc[55].value := dMnTdt;
tproc[56].value := dInnTdt;
tproc[57].value := dNminTdt;
tproc[58].value := dQnTdt;
tproc[59].value := dJdfsdt;
tproc[60].value := dJdwsdt;
tproc[61].value := dJdrsdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := Ta;
tproc[ModelDef.numstate + 2].value := Hd;
tproc[ModelDef.numstate + 3].value := Pptr;
tproc[ModelDef.numstate + 4].value := Ppts;
tproc[ModelDef.numstate + 5].value := PptI;
tproc[ModelDef.numstate + 6].value := Aw;
tproc[ModelDef.numstate + 7].value := PsiD;
tproc[ModelDef.numstate + 8].value := gcW;
tproc[ModelDef.numstate + 9].value := ET;
tproc[ModelDef.numstate + 10].value := Ms;
tproc[ModelDef.numstate + 11].value := DR;
tproc[ModelDef.numstate + 12].value := Qg;
tproc[ModelDef.numstate + 13].value := Nnh4aq;
tproc[ModelDef.numstate + 14].value := Nno3aq;
tproc[ModelDef.numstate + 15].value := Nnh4s;
tproc[ModelDef.numstate + 16].value := Nno3s;
tproc[ModelDef.numstate + 17].value := QNH4;
tproc[ModelDef.numstate + 18].value := QNO3;
tproc[ModelDef.numstate + 19].value := Cswreq;
tproc[ModelDef.numstate + 20].value := gcC;
tproc[ModelDef.numstate + 21].value := LAI;
tproc[ModelDef.numstate + 22].value := Ps;
tproc[ModelDef.numstate + 23].value := deltaPs;
tproc[ModelDef.numstate + 24].value := tauND;
tproc[ModelDef.numstate + 25].value := tauNlambda;
tproc[ModelDef.numstate + 26].value := HU;
tproc[ModelDef.numstate + 27].value := UNH4;
tproc[ModelDef.numstate + 28].value := UNO3;
tproc[ModelDef.numstate + 29].value := qUn;
tproc[ModelDef.numstate + 30].value := Unfix;
tproc[ModelDef.numstate + 31].value := RUnfix;
tproc[ModelDef.numstate + 32].value := HR;
tproc[ModelDef.numstate + 33].value := RFmet;
tproc[ModelDef.numstate + 34].value := RWmet;
tproc[ModelDef.numstate + 35].value := RRmet;
tproc[ModelDef.numstate + 36].value := RFgr;
tproc[ModelDef.numstate + 37].value := RWgr;
tproc[ModelDef.numstate + 38].value := RRgr;
tproc[ModelDef.numstate + 39].value := fL;
tproc[ModelDef.numstate + 40].value := RfmetL;
tproc[ModelDef.numstate + 41].value := RwmetL;
tproc[ModelDef.numstate + 42].value := RrmetL;
tproc[ModelDef.numstate + 43].value := RfmetS;
tproc[ModelDef.numstate + 44].value := RwmetS;
tproc[ModelDef.numstate + 45].value := RrmetS;
tproc[ModelDef.numstate + 46].value := tauNRf;
tproc[ModelDef.numstate + 47].value := tauNRsw;
tproc[ModelDef.numstate + 48].value := tauNRr;
tproc[ModelDef.numstate + 49].value := Pf;
tproc[ModelDef.numstate + 50].value := Pw;
tproc[ModelDef.numstate + 51].value := Pr;
tproc[ModelDef.numstate + 52].value := FGrow;
tproc[ModelDef.numstate + 53].value := WGrow;
tproc[ModelDef.numstate + 54].value := RGrow;
tproc[ModelDef.numstate + 55].value := Af;
tproc[ModelDef.numstate + 56].value := Ar;
tproc[ModelDef.numstate + 57].value := Gcf;
tproc[ModelDef.numstate + 58].value := Gcsw;
tproc[ModelDef.numstate + 59].value := Gchw;
tproc[ModelDef.numstate + 60].value := Gcr;
tproc[ModelDef.numstate + 61].value := Gnf;
tproc[ModelDef.numstate + 62].value := Gnsw;
tproc[ModelDef.numstate + 63].value := Gnhw;
tproc[ModelDef.numstate + 64].value := Gnr;
tproc[ModelDef.numstate + 65].value := lambdaCFD;
tproc[ModelDef.numstate + 66].value := lambdaCRD;
tproc[ModelDef.numstate + 67].value := lambdaNFD;
tproc[ModelDef.numstate + 68].value := lambdaNRD;
tproc[ModelDef.numstate + 69].value := FFall;
tproc[ModelDef.numstate + 70].value := WFall;
tproc[ModelDef.numstate + 71].value := RFall;
tproc[ModelDef.numstate + 72].value := lambdaCFs;
tproc[ModelDef.numstate + 73].value := lambdaCSWs;
tproc[ModelDef.numstate + 74].value := lambdaCHWs;
tproc[ModelDef.numstate + 75].value := lambdaCRs;
tproc[ModelDef.numstate + 76].value := lambdaNFs;
tproc[ModelDef.numstate + 77].value := lambdaNSWs;
tproc[ModelDef.numstate + 78].value := lambdaNHWs;
tproc[ModelDef.numstate + 79].value := lambdaNRs;
tproc[ModelDef.numstate + 80].value := lambdaCF;
tproc[ModelDef.numstate + 81].value := lambdaCSW;
tproc[ModelDef.numstate + 82].value := lambdaCHW;
tproc[ModelDef.numstate + 83].value := lambdaCR;
tproc[ModelDef.numstate + 84].value := lambdaNF;
tproc[ModelDef.numstate + 85].value := lambdaNSW;
tproc[ModelDef.numstate + 86].value := lambdaNHW;
tproc[ModelDef.numstate + 87].value := lambdaNR;
tproc[ModelDef.numstate + 88].value := lambdaCW;
tproc[ModelDef.numstate + 89].value := lambdaNW;
tproc[ModelDef.numstate + 90].value := FClambdaE;
tproc[ModelDef.numstate + 91].value := FClambdaS;
tproc[ModelDef.numstate + 92].value := FClambdaI;
tproc[ModelDef.numstate + 93].value := FNlambdaA;
tproc[ModelDef.numstate + 94].value := rSm;
tproc[ModelDef.numstate + 95].value := FCSE;
tproc[ModelDef.numstate + 96].value := FCEI;
tproc[ModelDef.numstate + 97].value := FCIE;
tproc[ModelDef.numstate + 98].value := FCIH;
tproc[ModelDef.numstate + 99].value := RE;
tproc[ModelDef.numstate + 100].value := RH;
tproc[ModelDef.numstate + 101].value := MNA;
tproc[ModelDef.numstate + 102].value := MNH;
tproc[ModelDef.numstate + 103].value := FNAH;
tproc[ModelDef.numstate + 104].value := INH4;
tproc[ModelDef.numstate + 105].value := INO3;
tproc[ModelDef.numstate + 106].value := Fntri;
tproc[ModelDef.numstate + 107].value := Cw;
tproc[ModelDef.numstate + 108].value := Nw;
tproc[ModelDef.numstate + 109].value := qF;
tproc[ModelDef.numstate + 110].value := qSW;
tproc[ModelDef.numstate + 111].value := qHW;
tproc[ModelDef.numstate + 112].value := qW;
tproc[ModelDef.numstate + 113].value := qR;
tproc[ModelDef.numstate + 114].value := qL;
tproc[ModelDef.numstate + 115].value := qSia;
tproc[ModelDef.numstate + 116].value := qSh;
tproc[ModelDef.numstate + 117].value := qrs;
tproc[ModelDef.numstate + 118].value := CTP;
tproc[ModelDef.numstate + 119].value := CTSa;
tproc[ModelDef.numstate + 120].value := CTS;
tproc[ModelDef.numstate + 121].value := NTP;
tproc[ModelDef.numstate + 122].value := NTS;
tproc[ModelDef.numstate + 123].value := qTP;
tproc[ModelDef.numstate + 124].value := qTS;
tproc[ModelDef.numstate + 125].value := NTI;
tproc[ModelDef.numstate + 126].value := CTE;
tproc[ModelDef.numstate + 127].value := NTE;
tproc[ModelDef.numstate + 128].value := RF;
tproc[ModelDef.numstate + 129].value := RW;
tproc[ModelDef.numstate + 130].value := RR;
tproc[ModelDef.numstate + 131].value := Rp;
tproc[ModelDef.numstate + 132].value := Rs;
tproc[ModelDef.numstate + 133].value := NPPf;
tproc[ModelDef.numstate + 134].value := NPPw;
tproc[ModelDef.numstate + 135].value := NPPr;
tproc[ModelDef.numstate + 136].value := NPP;
tproc[ModelDef.numstate + 137].value := NEP;
tproc[ModelDef.numstate + 138].value := lambdaC;
tproc[ModelDef.numstate + 139].value := lambdaN;
tproc[ModelDef.numstate + 140].value := Dn;
tproc[ModelDef.numstate + 141].value := tauN;
tproc[ModelDef.numstate + 142].value := Uni;
tproc[ModelDef.numstate + 143].value := Un;
tproc[ModelDef.numstate + 144].value := Mn;
tproc[ModelDef.numstate + 145].value := Inn;
tproc[ModelDef.numstate + 146].value := Nmin;
tproc[ModelDef.numstate + 147].value := Qn;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t, drt, tdrive, tpar, tempstate, tempproc, false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
